/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 *  Merja.java
 *  2003-2004 Darnet Camille, Vouriot Thierry
 *  camsss@ifrance.com, yeri@fr.st
 *  http://www.yeri.fr.st
 *
 */
package merja;

import java.awt.*;
import java.io.*;
import java.util.*;
import javax.swing.plaf.metal.MetalLookAndFeel;
import javax.swing.*;
import merja.gui.*;
import merja.gui.actions.*;

/** Class principale
 */
public class Merja {
    
    /** Version */
    public static final String RELEASE = "1.2";
    
    /** Build */
    public static final String BUILD = "040131";
    
    /** Site web */
    public static final String WEB = "http://www.yeri.fr.st";
    
    /** Mode "debug" */
    public static final boolean DEBUG = false;
    
    /** Rpertoire home */
    public static final File HOME_DIRECTORY = new File(System.getProperty("user.home"));
    
    /** Table contenant toutes les actions */
    private static final HashMap m_ActionHash = new HashMap();
    
    /** Chemin d'accs du rpertoire o est install Elife */
    private static String m_MerjaHome = null;
    
    /** Ajouter une action  la table
     * @param action action listener
     */
    public static void addAction(MerjaAction action) {
        m_ActionHash.put(action.getName(), action);
    }
    
    /** Retourne une action  partir de son nom
     * @param action nom de l'action
     * @return l'action
     */
    public static MerjaAction getAction(String action) {
        return (MerjaAction) m_ActionHash.get(action);
    }
    
    /** Initialise la table des actions en y ajoutant toutes les actions possibles
     */
    public static void initActions() {
        addAction(new ActionStop());
        addAction(new ActionAbout());
        addAction(new ActionOpen());
        addAction(new ActionOpenDB());
        addAction(new ActionAssociation());
        addAction(new ActionDelete());
        addAction(new ActionItemsets());
    }
    
    /** Initialise le look & feel Metal
     */
    public static void initUI() {
        MetalLookAndFeel.setCurrentTheme(new MerjaMetalTheme());
    }
    
    /** Retourne le chemin d'accs du rpertoire o est install Elife
     *  @return chemin d'accs d'Elife
     */
    public static String getMerjaHome() {
        
        if (m_MerjaHome == null) {
            String classpath = System.getProperty("java.class.path");
            int index = classpath.toLowerCase().indexOf("merja.jar");
            int start = classpath.lastIndexOf(File.pathSeparator,index) + 1;
            // if started with java -jar Merja.jar
            if(classpath.equalsIgnoreCase("merja.jar")) {
                m_MerjaHome = System.getProperty("user.dir");
            }
            else if(index > start) {
                m_MerjaHome = classpath.substring(start,index - 1);
            }
            else m_MerjaHome = System.getProperty("user.dir");
        }
        return m_MerjaHome;
    }
    
    /** Point de dpart de l'application
     * @param argv arguments de la ligne de commande
     */
    public static void main(String [] argv) {
        MerjaFrame frm = new MerjaFrame();
        frm.show();
        frm.validate();
        frm.repaint();
    }
}
