/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 *  GeneItems.java
 *  2003-2004 Darnet Camille, Vouriot Thierry
 *  camsss@fr.st, yeri@fr.st
 *  http://www.yeri.fr.st
 *
 */
package merja.associations.itemFreq;

import merja.associations.Item;
import merja.associations.Hash;
import merja.associations.Transaction;
import java.util.Vector;

public class GeneItems {
    /**
     * Support minimun utilis lors de la suppression des items candidats
     * <br> remarque : utiliser avec prcautions
     */
    protected static int supportMin;
    /**
     * Hauteur de l'arbre, surtout utilis pour les parcours
     * <br> remarque : utiliser avec prcautions
     */
    protected static int hauteur;
    /**
     * Premier noeud dans l'arbre
     * <br> remarque : utiliser avec prcautions
     */
    protected static NoeudItem noeud0;
    /**
     * Table de correspodance
     * <br> remarque : utiliser avec prcautions
     */
    protected static Hash hash;
    /**
     * Donnes statistique sur le nombre de noeud
     * <br> remarque : peut -tre manipul n'importe comment a drange pas l'algo
     */
    public static int nbNoeud=0;
    /**
     * Donnes statistique sur le nombre d'itemset frquents
     * <br> remarque : peut -tre manipul n'importe comment a drange pas l'algo
     */
    public static int nbIdItemSupport=0;
    
    /**
     * Cr une instance de GeneItem
     * <br> cette classe permet de manipuler l'arbre des ensembles des items frquens
     * sans rentrer dans les dtails de la programmation
     * @param hash : table de correspondance entre les items et leurs identifiants
     * @param supportMin : Support minimum utilis lors de la suppresion des ensembles
     * candidats
     */
    public GeneItems(Hash hash,int supportMin){
        this.hash=hash;
        hauteur=0;
        this.supportMin=supportMin;
        nbNoeud=0;
        nbIdItemSupport=0;
        nbNoeud=1;
        noeud0=new NoeudItem(hash.getLength());
        for(int i=0;i<hash.getLength();i++){
            noeud0.add(i);
        }
    }
    
    /**
     *
     * @return l'ensemble des items frquents
     */
    public String toString(){
        return noeud0.toString(0,"");
    }
    
    /**
     * Obsoleste
     * Retourne tous les items Frequents sous forme de string dans un vector
     * @return vector de String
     */
    
    public Vector getItemsFrequentsString(){
        Vector resu=new Vector();
        noeud0.addItem(resu,0,"");
        for(int i=0;i<resu.size();i++){
            System.out.println(((String)(resu.get(i))));
        }
        
        return resu;
    }
    
    /**
     * supprime les ensembles ont support insuffisant
     */
    public void removes(){
        noeud0.removes(0);
    }
    
    /**
     * Calcule le support des ensembles appartenant a cette transaction
     * <br> Le support est calcul seulement pour les nouveaus ensembles candidtas
     * @param transaction ben une transaction
     */
    public void calculSupport(Transaction transaction){
        int[] trans = transaction.getItemId();
        if (trans.length>hauteur)
            noeud0.calculSupport(trans,0);
    }
    
    public Vector getItemsSetFrequents(){
        Vector resu=new Vector();
        noeud0.addItemSet(resu,new ItemSet());
        return resu;
    }
    
    /**
     *
     * @return tous les ensembles d'items frquents
     */
    public Vector getItemsFrequents(){
        Vector itemsFreq=new Vector();
        noeud0.addItemsFrequent(itemsFreq,new int[hauteur+2],0);
        
        return itemsFreq;
    }
    
    /**
     * Permet de retrouver le support d'un ensemble frquents
     * <br> ATTENTION il faut que l'ensemble d'items frquents soit dans l'arbre
     * sinon a PLANTE !!!!!!
     * @param listeItems corresondant  un ensemble d'items frquents
     * @return le support
     */
    public int getSupport(int[] listeItems){
        return noeud0.getSupport(listeItems,0);
    }
    
    /**
     * Cration des ensembles candidats  partir des ensembles frquents
     * @return true si des nouveaux lments sont crs
     * <br> false sinon
     */
    public boolean jointure(){
        int nbNoeudOld=nbNoeud;
        noeud0.jointure(0,new int[GeneItems.hauteur+1]);
        GeneItems.hauteur++;
        return nbNoeudOld!=nbNoeud;
    }
    
    /**
     *
     * @return le nombre d'items
     */
    protected int getNbItems(){
        return nbIdItemSupport;
    }
    
    public static void main(String[] args) {
        Item[] itemList = new Item[5];
        itemList[0] = new Item("", "5");
        itemList[1] = new Item("", "2");
        itemList[2] = new Item("", "3");
        itemList[3] = new Item("", "4");
        itemList[4] = new Item("", "1");
        Hash h = new Hash(itemList);
        
        Item[] itemList_ = new Item[3];
        itemList_[0] = new Item("", "4");
        itemList_[1] = new Item("", "3");
        itemList_[2] = new Item("", "1");
        
        Item[] itemList__ = new Item[3];
        itemList__[0] = new Item("", "5");
        itemList__[1] = new Item("", "3");
        itemList__[2] = new Item("", "2");
        
        Item[] itemList_1 = new Item[4];
        itemList_1[0] = new Item("", "5");
        itemList_1[1] = new Item("", "3");
        itemList_1[2] = new Item("", "2");
        itemList_1[3] = new Item("", "1");
        
        Item[] itemList_2 = new Item[2];
        itemList_2[0] = new Item("", "5");
        itemList_2[1] = new Item("", "2");
        
        Transaction trans = new Transaction(h);
        trans.addItemList(itemList_);
        Transaction trans1 = new Transaction(h);
        trans1.addItemList(itemList__);
        Transaction trans2 = new Transaction(h);
        trans2.addItemList(itemList_1);
        Transaction trans3 = new Transaction(h);
        trans3.addItemList(itemList_2);
        
        System.out.println(h.toString());
        GeneItems gene = new GeneItems(h,2);
        
        gene.calculSupport(trans);
        gene.calculSupport(trans1);
        gene.calculSupport(trans2);
        gene.calculSupport(trans3);
        
        noeud0.jointure(0,new int[GeneItems.hauteur+1]);
        GeneItems.hauteur++;
        gene.calculSupport(trans);
        gene.calculSupport(trans1);
        gene.calculSupport(trans2);
        gene.calculSupport(trans3);
        
        //System.out.println(noeud0.isValide(new int[]{0,1,0},0));
        noeud0.jointure(0,new int[GeneItems.hauteur+1]);
        GeneItems.hauteur++;
        System.out.println(gene.toString());
        gene.calculSupport(trans);
        gene.calculSupport(trans1);
        gene.calculSupport(trans2);
        gene.calculSupport(trans3);
        System.out.println("\n"+gene.toString());
        noeud0.jointure(0,new int[GeneItems.hauteur+1]);
        GeneItems.hauteur++;
        System.out.println(gene.toString());
        System.out.println(gene.nbNoeud);
    }
}