/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 *  AboutDialog.java
 *  2003-2004 Darnet Camille, Vouriot Thierry
 *  camsss@ifrance.com, yeri@fr.st
 *  http://www.yeri.fr.st
 *
 */
package merja.gui;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import merja.*;

/** Fentre <I>A propos ...</I>
 */
public final class AboutDialog extends JDialog {
    
    /** Constructeur
     *  @param fr fentre principale de l'application
     */
    public AboutDialog(JFrame fr) {
        super(fr,"A Propos de Merja",true);
        this.setSize(370,245);
        this.setLocation(Utilities.getCenterComponentChild(fr,this));
        
        Container contentPane = this.getContentPane();
        
        JTextArea txt = new JTextArea();
        String s = "Java Environment for Mining Association Rules\n";
        s += "---------------------------------------------\n";
        s += "Projet ralis dans le cadre du\nDESS Ingnieurie Informatique de Strasbourg\n";
        s += "sous la supervision de Mr Jerzy Korczak\n(jjk@dpt-info.u-strasbg.fr)\n\n";
        s += "Auteurs: Camille Darnet, Thierry Vouriot\n";
        s += "Emails: camsss@fr.st, yeri@fr.st\n";
        s += "Web: "+Merja.WEB+"";
        JLabel lblRelease = new JLabel("version "+Merja.RELEASE+" (build "+Merja.BUILD+")",JLabel.RIGHT);
        
        Font f1 = new Font("Monospaced",Font.PLAIN,11);
        Font f2 = new Font("Arial",Font.ITALIC,10);
        
        JPanel pnlAbout = new JPanel(new BorderLayout());
        JPanel pnlSystem = new JPanel(new BorderLayout());
        JTabbedPane tabs = new JTabbedPane();
        String properties[][] = new String[12][2];
        
        properties[0][0] = "Product Version :";
        properties[0][1] = "Merja v"+Merja.RELEASE+" (build "+Merja.BUILD+")";
        
        properties[1][0] = "Product Date :";
        properties[1][1] = Merja.BUILD.substring(4,6)+"/"+Merja.BUILD.substring(2,4)+"/20"+Merja.BUILD.substring(0,2);
        
        properties[2][0] = "OS Name :";
        properties[2][1] = System.getProperty("os.name");
        
        properties[3][0] = "OS Version :";
        properties[3][1] = System.getProperty("os.version");
        
        properties[4][0] = "Architecture :";
        properties[4][1] = System.getProperty("os.arch");
        
        properties[5][0] = "Java :";
        properties[5][1] = System.getProperty("java.version");
        
        properties[6][0] = "Vendor :";
        properties[6][1] = System.getProperty("java.vendor");
        
        properties[7][0] = "Vendor Web :";
        properties[7][1] = System.getProperty("java.vendor.url");
        
        properties[8][0] = "Java Home :";
        properties[8][1] = System.getProperty("java.home");
        
        properties[9][0] = "User Name :";
        properties[9][1] = System.getProperty("user.name");
        
        properties[10][0] = "Home Dir :";
        properties[10][1] = System.getProperty("user.home");
        
        properties[11][0] = "Current Dir :";
        properties[11][1] = System.getProperty("user.dir");
        
        JTable table = new JTable(properties, new String[]{"",""});
        table.setEnabled(false);
        table.setShowVerticalLines(false);
        table.setOpaque(true);
        tabs.addTab("Infos",pnlAbout);
        tabs.addTab("System",pnlSystem);
        tabs.setFont(f2);
        
        txt.setFont(f1);
        lblRelease.setFont(f1);
        
        txt.setEditable(false);
        txt.setBackground(pnlAbout.getBackground());
        txt.setText(s);
        
        pnlAbout.setBorder(BorderFactory.createEmptyBorder(5, 5, 0, 5));
        pnlSystem.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
        
        pnlAbout.add(new JScrollPane(txt),BorderLayout.CENTER);
        pnlAbout.add(lblRelease,BorderLayout.SOUTH);
        pnlSystem.add(new JScrollPane(table),BorderLayout.CENTER);
        contentPane.add(tabs);
        
        this.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                dispose();
            }
        });
        
        txt.addMouseListener(new MouseAdapter() {
            public void mouseClicked(final java.awt.event.MouseEvent p1) {
                dispose();
            }
        });
    }
}
