/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 *  DBChooserDialog.java
 *  2003-2004 Darnet Camille, Vouriot Thierry
 *  camsss@fr.st, yeri@fr.st
 *  http://www.yeri.fr.st
 *
 */
package merja.gui;

import java.lang.*;
import java.util.*;
import java.sql.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.text.DefaultStyledDocument;
import javax.swing.border.*;
import javax.swing.table.*;
import merja.*;
import merja.db.*;

/** Fentre permettant de charger des drivers JDBC, d'ouvrir une base de donnes et
 * de slectionner une table avec un certain nombre d'attributs
 */
public final class DBChooserDialog extends JDialog {
    
    private final JComboBox m_cmbDriver = new JComboBox(new String[] {"sun.jdbc.odbc.JdbcOdbcDriver",
    "com.mysql.jdbc.Driver",
    "org.postgresql.Driver"});
    private final JTextField m_txtURL = new JTextField("jdbc:odbc:");
    private final JTextField m_txtUser = new JTextField();
    private final JTextField m_txtPassword = new JTextField();
    private final JLabel m_lblStatusConnection = new JLabel(" ");
    private final JComboBox m_cmbTables = new JComboBox();
    private final JTable m_tblAttributs = new JTable();
    private final JButton m_cmdOk = new JButton("Valider");
    private final JButton m_cmdCancel = new JButton("Annuler");
    private final Database m_db = new Database();
    private int returnValue = JFileChooser.ERROR_OPTION;
    private final JTabbedPane m_tabbedPane = new JTabbedPane();
    private final JEditorPane m_txtSql = new JEditorPane();
    private final JFileChooser m_fc = new JFileChooser(Merja.getMerjaHome());
    
    /** Constructeur
     *  @param fr fentre principale de l'application
     */
    public DBChooserDialog(final JFrame fr) {
        super(fr,"Ouverture BDD",true);
        this.setSize(360,400);
        this.setLocation(Utilities.getCenterComponentChild(fr,this));
        
        Container contentPane = this.getContentPane();
        
        // ================ Panel Paramtres ==========================
        JPanel panelParams = new JPanel(new GridLayout(5,1,10,3));
        panelParams.setBorder(BorderFactory.createCompoundBorder(
        BorderFactory.createTitledBorder("Paramtres"),
        BorderFactory.createEmptyBorder(0, 0, 0, 0)
        ));
        
        JPanel panelDriver = new JPanel(new BorderLayout(5,1));
        m_cmbDriver.setEditable(true);
        JLabel lblDriver = new JLabel("Driver :");
        lblDriver.setPreferredSize(new Dimension(60, 10));
        panelDriver.add(lblDriver, BorderLayout.WEST);
        panelDriver.add(m_cmbDriver, BorderLayout.CENTER);
        
        JPanel panelURL = new JPanel(new BorderLayout(5,1));
        JLabel lblURL = new JLabel("URL :");
        lblURL.setPreferredSize(new Dimension(60, 10));
        panelURL.add(lblURL, BorderLayout.WEST);
        panelURL.add(m_txtURL, BorderLayout.CENTER);
        
        JPanel panelUser = new JPanel(new BorderLayout(5,1));
        JLabel lblUser = new JLabel("Utilisateur :");
        lblUser.setPreferredSize(new Dimension(60, 10));
        panelUser.add(lblUser, BorderLayout.WEST);
        panelUser.add(m_txtUser, BorderLayout.CENTER);
        
        JPanel panelPassword = new JPanel(new BorderLayout(5,1));
        JLabel lblPassword = new JLabel("Password :");
        lblPassword.setPreferredSize(new Dimension(60, 10));
        panelPassword.add(lblPassword, BorderLayout.WEST);
        panelPassword.add(m_txtPassword, BorderLayout.CENTER);
        
        JPanel panelConnection = new JPanel(new FlowLayout(FlowLayout.RIGHT,10,0));
        JButton cmdConnection = new JButton("Connexion");
        cmdConnection.setPreferredSize(new Dimension(90, 23));
        //m_lblStatusConnection.setPreferredSize(new Dimension(200, 20));
        panelConnection.add(m_lblStatusConnection);
        panelConnection.add(cmdConnection);
        
        panelParams.add(panelDriver);
        panelParams.add(panelURL);
        panelParams.add(panelUser);
        panelParams.add(panelPassword);
        panelParams.add(panelConnection);
        
        cmdConnection.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                System.out.println("Action: connection");
                connection();
            }
        });
        
        m_cmbDriver.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                if (((String)m_cmbDriver.getSelectedItem()).equals("sun.jdbc.odbc.JdbcOdbcDriver"))
                    m_txtURL.setText("jdbc:odbc:");
                else if (((String)m_cmbDriver.getSelectedItem()).equals("com.mysql.jdbc.Driver"))
                    m_txtURL.setText("jdbc:mysql:");
                else if (((String)m_cmbDriver.getSelectedItem()).equals("org.postgresql.Driver"))
                    m_txtURL.setText("jdbc:postgresql:");
            }
        });
        // ============================================================
        
        // ================ Panel Tables et Attributs =================
        JPanel panelTablesAtt = new JPanel(new BorderLayout(1,5));
        panelTablesAtt.setBorder(BorderFactory.createEmptyBorder(2, 2, 2, 2));
        
        JPanel panelTables = new JPanel(new BorderLayout(5,1));
        JLabel lblTables = new JLabel("Tables :");
        lblTables.setPreferredSize(new Dimension(60, 10));
        panelTables.add(lblTables, BorderLayout.WEST);
        panelTables.add(m_cmbTables, BorderLayout.CENTER);
        
        JPanel panelAttributs = new JPanel(new BorderLayout(5,3));
        JLabel lblAttributs = new JLabel("Attributs :");
        panelAttributs.add(lblAttributs, BorderLayout.NORTH);
        panelAttributs.add(new JScrollPane(m_tblAttributs), BorderLayout.CENTER);
        
        panelTablesAtt.add(panelTables, BorderLayout.NORTH);
        panelTablesAtt.add(panelAttributs, BorderLayout.CENTER);
        
        enableTablesAttributsSql(false);
        
        m_cmbTables.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                updateAttributs();
            }
        });
        // ============================================================
        
        // ================ Panel SQL =================================
        JPanel panelSql = new JPanel(new BorderLayout(1,5));
        panelSql.setBorder(BorderFactory.createEmptyBorder(2, 2, 2, 2));
        
        JPanel panelChargerSql = new JPanel(new FlowLayout(FlowLayout.RIGHT,10,0));
        JButton cmdChargerSql = new JButton("Charger ...");
        cmdChargerSql.setPreferredSize(new Dimension(90, 23));
        panelChargerSql.add(cmdChargerSql);
        
        panelSql.add(new JScrollPane(m_txtSql), BorderLayout.CENTER);
        panelSql.add(panelChargerSql, BorderLayout.SOUTH);
        enableTablesAttributsSql(false);
        
        final ExampleFileFilter filter = new ExampleFileFilter();
        filter.addExtension("sql");
        filter.setDescription("SQL script");
        m_fc.setFileFilter(filter);
        cmdChargerSql.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                try {
                    if (m_fc.showOpenDialog(fr) == JFileChooser.APPROVE_OPTION) {
                        m_txtSql.setDocument(new DefaultStyledDocument());
                        m_txtSql.setPage(m_fc.getSelectedFile().toURL());
                    }
                }
                catch (Exception e) { System.out.println("DBChooserDialog::cmdChargerSql.actionPerformed : "+e); }
            }
        });
        
        m_txtSql.setFont(new Font("Monospaced", Font.PLAIN, 12));
        
        // ============================================================
        
        m_cmdOk.setEnabled(false);
        
        JPanel panelSouth = new JPanel(new FlowLayout(FlowLayout.RIGHT, 10, 6));
        panelSouth.add(m_cmdOk);
        panelSouth.add(m_cmdCancel);
        
        m_tabbedPane.addTab("Tables et attributs", panelTablesAtt);
        m_tabbedPane.addTab("Requte SQL", panelSql);
        
        contentPane.add(panelParams, BorderLayout.NORTH);
        contentPane.add(m_tabbedPane, BorderLayout.CENTER);
        contentPane.add(panelSouth, BorderLayout.SOUTH);
        
        this.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                returnValue = JFileChooser.CANCEL_OPTION;
                dispose();
                
            }
        });
        
        m_cmdCancel.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                returnValue = JFileChooser.CANCEL_OPTION;
                dispose();
            }
        });
        
        m_cmdOk.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                returnValue = JFileChooser.APPROVE_OPTION;
                if (getAttributsNames().size() < 2 && m_tabbedPane.getSelectedIndex() == 0)
                    JOptionPane.showMessageDialog(fr,"Pour effectuer les calculs, il faut au moins 2 attributs slectionns !","Attention",JOptionPane.ERROR_MESSAGE);
                else if (m_cmbTables.getItemCount() < 1)
                    JOptionPane.showMessageDialog(fr,"Aucune table dans la base de donnes !","Attention",JOptionPane.ERROR_MESSAGE);
                else dispose();
            }
        });
    }
    
    /* Connection  la base de donnes */
    private void connection() {
        if (m_db.isConnect()) m_db.disconnect();
        try {
            // chargement du driver
            Database.loadDriver((String)m_cmbDriver.getSelectedItem());
            // connection  la base
            m_db.connect(m_txtURL.getText().trim(), m_txtUser.getText().trim(), m_txtPassword.getText().trim(), Database.JDBC1);
            this.m_lblStatusConnection.setText("Connexion russie");
            this.m_lblStatusConnection.setForeground(new Color(0.0f, 0.0f, 1.0f));
            this.m_lblStatusConnection.setToolTipText("");
            this.m_cmbTables.setModel(new DefaultComboBoxModel(m_db.getTablesNames()));
            enableTablesAttributsSql(true);
            updateAttributs();
        }
        // erreur de driver
        catch (ClassNotFoundException e) {
            this.m_lblStatusConnection.setText("Echec de connexion (Driver)");
            this.m_lblStatusConnection.setForeground(new Color(0.9f, 0.0f, 0.0f));
            this.m_lblStatusConnection.setToolTipText("Class not found : "+e.getMessage());
            enableTablesAttributsSql(false);
        }
        // erreur de connection
        catch (SQLException e) {
            this.m_lblStatusConnection.setText("Echec de connexion (Url, user ou pass)");
            this.m_lblStatusConnection.setForeground(new Color(0.9f, 0.0f, 0.0f));
            this.m_lblStatusConnection.setToolTipText(e.getMessage());
            enableTablesAttributsSql(false);
        }
    }
    
    /* Active ou desactive le panel "Table-Attributs" */
    private void enableTablesAttributsSql(boolean b) {
        m_tblAttributs.setEnabled(b);
        m_cmbTables.setEnabled(b);
        m_cmdOk.setEnabled(b);
        
        if (b == false) {
            this.m_cmbTables.setModel(new DefaultComboBoxModel());
            this.m_tblAttributs.setModel(new DefaultTableModel());
        }
    }
    
    /* Maj de la liste des attributs en fonction de la table slectionne */
    private void updateAttributs() {
        
        String[][] atts = m_db.getAttributes((String)m_cmbTables.getSelectedItem());
        
        if (atts != null) {
            Object[][] lines = new Object[atts.length][3];
            
            for (int i=0; i<atts.length; i++) {
                if (atts[i][1].equalsIgnoreCase("varchar"))
                    lines[i][0] = new Boolean(true);
                else lines[i][0] = new Boolean(false);
                lines[i][1] = atts[i][0];
                lines[i][2] = atts[i][1];
            }
            
            String[] cols = new String[] {" ", "Attribut", "Type"};
            AttributsTableModel model = new AttributsTableModel(lines, cols);
            
            this.m_tblAttributs.setModel(model);
            this.m_tblAttributs.getColumn(" ").setResizable(false);
            this.m_tblAttributs.getColumn(" ").setPreferredWidth(25);
            this.m_tblAttributs.getColumn(" ").setMaxWidth(25);
            this.m_tblAttributs.getColumn(" ").setMinWidth(25);
            this.m_tblAttributs.getColumn(" ").setHeaderRenderer(null);
        }
        else {
            DefaultTableModel model = new DefaultTableModel();
            this.m_tblAttributs.setModel(model);
        }
    }
    
    /** Affiche la fentre
     * @return <CODE>JFileChooser.APPROVE_OPTION</CODE> si le bouton valider a t slectionn et <CODE>JFileChooser.APPROVE_OPTION</CODE>
     * sinon
     * @param fr fentre principale de l'application
     */
    public int showOpenDialog(final JFrame fr) {
        this.setLocation(Utilities.getCenterComponentChild(fr,this));
        this.show();
        return returnValue;
    }
    
    /** Retourne l'objet <CODE>Database</CODE>
     * @return objet <CODE>Database</CODE>
     */
    public Database getDB() {
        return m_db;
    }
    
    /** Retourne le nom de la table slectionne dans la base de donnes ou "requte SQL"
     * @return nom de la table ou "requte SQL"
     */
    public String getName() {
        String res = "Requte SQL";
        if (this.m_tabbedPane.getSelectedIndex() == 1)
            return res;
        else return (String)(this.m_cmbTables.getSelectedItem());
    }
    
    /** Retourne la liste des attributs slectionns dans la table
     * @return liste des attributs
     */
    public Vector getAttributsNames() {
        Vector v = new Vector();
        for (int i=0; i<m_tblAttributs.getRowCount(); i++)
            if (((Boolean)(m_tblAttributs.getValueAt(i,0))).booleanValue())
                v.addElement(m_tblAttributs.getValueAt(i,1));
        return v;
    }
    
    /** Retourne la requte SQL correspondant  la table et aux attributs choisis ou  la requte spcifie
     * @return requte SQL
     */
    public String getSqlQuery() {
        if (this.m_tabbedPane.getSelectedIndex() == 0) {
            StringBuffer query = new StringBuffer("SELECT `");
            for (int i=0; i<m_tblAttributs.getRowCount(); i++)
                if (((Boolean)(m_tblAttributs.getValueAt(i,0))).booleanValue())
                    query.append((String)(m_tblAttributs.getValueAt(i,1)) + "`,`");
            query.delete(query.length()-3, query.length());
            query.append("` FROM `" + (String)(this.m_cmbTables.getSelectedItem()) + "` WHERE 1");
            
            return query.toString().toLowerCase();
        }
        else {
            String query = m_txtSql.getText().trim().toLowerCase();
            // On ajoute une clause "where" si il n'y en a pas
            StringBuffer buf = new StringBuffer(query);
            int i = query.lastIndexOf("where");
            if (i==-1) {
                i = query.length();
                buf.insert(i, " where 1");
            }
            return buf.toString();
        }
    }
    
    private class AttributsTableModel extends DefaultTableModel {
        private static final int SELECTION = 0;
        private static final int TABLENAME = 1;
        private static final int TYPE = 2;
        
        public AttributsTableModel(Object [][] data, Object[] columnNames) {
            super(data, columnNames);
        }
        
        public Class getColumnClass(int columnIndex) {
            if (columnIndex == SELECTION) return Boolean.class;
            return Object.class;
        }
        
        public boolean isCellEditable(int rowIndex, int columnIndex) {
            if (columnIndex == SELECTION) return true;
            return false;
        }
    }
}





