/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 *  ItemsetsPanel.java
 *  2003-2004 Darnet Camille, Vouriot Thierry
 *  camsss@ifrance.com, yeri@fr.st
 *  http://www.yeri.fr.st
 *
 */
package merja.gui;

import java.util.*;
import java.awt.event.*;
import java.awt.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import merja.*;
import merja.images.Icons;
import merja.associations.*;
import merja.associations.itemFreq.*;

/** Panel permettant le calcul et l'affichage des itemsets frquents */
public class ItemsetsPanel extends MerjaPanel {
    
    private final JList m_itemsetsList = new JList();
    private final JSlider m_supportSlider = new JSlider(JSlider.HORIZONTAL,1,100,50);
    private final JLabel m_supportLbl = new JLabel();
    private final JButton m_startBtn = new JButton("Extraire les itemsets");
    private final JButton m_stopBtn = new JButton("Stop");
    private final JButton m_saveBtn = new JButton("Save output");
    private final LogTextArea m_log = new LogTextArea();
    
    /** Constructeur */
    ItemsetsPanel() {
        this.setLayout(new BorderLayout());
        
        JPanel panelWest = new JPanel(new BorderLayout());
        JPanel panelCenter = new JPanel(new BorderLayout());
        
       // m_itemsetsList.setEditable(false);
        m_stopBtn.setEnabled(false);
        
        panelCenter.add(new JScrollPane(m_itemsetsList),BorderLayout.CENTER);
        panelCenter.setBorder(BorderFactory.createCompoundBorder(
        BorderFactory.createTitledBorder("Itemsets sortie"),
        BorderFactory.createEmptyBorder(0, 0, 0, 0)
        ));
        
        final JLabel supportLabel = new JLabel("50%");
        JPanel panelSupport = new JPanel(new BorderLayout());
        m_supportSlider.setMinorTickSpacing(5);
        m_supportSlider.setMajorTickSpacing(10);
        m_supportSlider.setPaintTicks(true);
        m_supportSlider.setPaintTrack(true);
        m_supportSlider.setPreferredSize(new Dimension(110, 30));
        supportLabel.setPreferredSize(new Dimension(30, 15));
        panelSupport.add(m_supportSlider,BorderLayout.CENTER);
        panelSupport.add(supportLabel,BorderLayout.EAST);
        panelSupport.add(m_supportLbl,BorderLayout.SOUTH);
        panelSupport.setBorder(BorderFactory.createCompoundBorder(
        BorderFactory.createTitledBorder("Support minimum"),
        BorderFactory.createEmptyBorder(0, 0, 0, 0)
        ));
        
        JPanel panelStartStop = new JPanel(new GridLayout(2,1,4,2));
        panelStartStop.add(m_startBtn);
        panelStartStop.add(m_stopBtn);
        panelStartStop.setBorder(BorderFactory.createEmptyBorder(3, 3, 3, 1));
        
        panelWest.add(panelSupport,BorderLayout.NORTH);
        panelWest.add(m_log,BorderLayout.CENTER);
        panelWest.add(panelStartStop,BorderLayout.SOUTH);
        
        this.add(panelWest,BorderLayout.WEST);
        this.add(panelCenter,BorderLayout.CENTER);
        
        m_supportSlider.addChangeListener(new ChangeListener() {
            public void stateChanged(ChangeEvent evt) {
                setSupportLabel();
                String s = m_supportSlider.getValue()+"%";
                supportLabel.setText(s);
            }
        });
        
        m_startBtn.addActionListener(new ActionListener() {
            public void actionPerformed(final java.awt.event.ActionEvent evt) {
                calcItemsFreq();
            }
        });
        
        m_stopBtn.addActionListener(new ActionListener() {
            public void actionPerformed(final java.awt.event.ActionEvent evt) {
                stopThread();
                enableDisableButtons();
            }
        });
        
        m_itemsetsList.addMouseListener(new MouseAdapter() {
            public void mouseClicked(final java.awt.event.MouseEvent evt) {
                if (evt.getClickCount() == 2 && m_merjaFrame.getFileDBPanel().getDataType()==FileDBPanel.DB) {  
                    Point p = new Point(evt.getX(), evt.getY());
                    SwingUtilities.convertPointToScreen(p,  evt.getComponent());
                    ItemSet is = (ItemSet)(m_itemsetsList.getModel().getElementAt(m_itemsetsList.locationToIndex(evt.getPoint())));
                    Item[] items = is.getItems();
                    new DBItemsetFrame(m_merjaFrame, items, is.toString(), p).show();
                }
            }
        });
    }
    
    /** Active ou desactive les boutons <I>start</I> et <I>stop</I> suivant qu'il y ai
     * un traitement en cours ou non
     */        
    public void enableDisableButtons() {
        m_stopBtn.setEnabled(m_runThread != null);
        m_startBtn.setEnabled(m_runThread == null);
    }
    
    /** Calcule et affiche les itemstes frquents */    
    public void calcItemsFreq() {
        if (m_runThread == null) {
            m_runThread = new Thread("<itemsets>") {
                public void run() {
                    enableDisableButtons();
                    m_log.clean();
                    m_merjaFrame.getAlgoInterface().setLog(m_log);
                    m_merjaFrame.activePanelsAndButtons(true, true, true, false);
                    int support = (int)((float)(getSupport()/100.0) * (float)(m_merjaFrame.getTR().getNbTransactions()));
                    if (support == 0) support = 1;
                    m_logPanel.taskStarted();
                    m_logPanel.logMessage("DEBUT : Calcul des itemsets frquents");
                    m_itemsetsList.setListData(new Vector());
                    m_merjaFrame.getAssociationRulesPanel().cleanAll();
                    m_merjaFrame.getChrono().start();
                    m_merjaFrame.getAlgoInterface().setSupportMin(support);
                    m_merjaFrame.getAlgoInterface().geneItemFreq();
                    m_merjaFrame.getItemsetsPanel().setItemsets(m_merjaFrame.getAlgoInterface().getItemSetFreq());
                    m_merjaFrame.getChrono().stop();
                    m_logPanel.logMessage("FIN : Calcul des itemsets frquents (" + m_merjaFrame.getChrono().timeElapsed() + "ms) : support = " + support);
                    m_logPanel.taskFinished();
                    m_runThread = null;
                    m_merjaFrame.activePanelsAndButtons(true, true, true, true);
                    enableDisableButtons();
                }
            };
            m_runThread.setPriority(Thread.MIN_PRIORITY);
            m_runThread.start();
        }   
    }
    
    /** Change le label du support minimum pour afficher le nombre de transactions
     * correspondant au pourcentage du <I>slider</I>
     */    
    public void setSupportLabel() {
        if (m_merjaFrame.getTR() != null) {
            int support = (int)((float)(m_supportSlider.getValue()/100.0) * (float)(m_merjaFrame.getTR().getNbTransactions()));
            m_supportLbl.setText("  "+support +"/"+ m_merjaFrame.getTR().getNbTransactions() + " transactions");
        }
        else m_supportLbl.setText("");
    }
    
    private void setItemsets(Vector v) {
        m_itemsetsList.setListData(v);
    }
    
    private int getSupport() {
        return this.m_supportSlider.getValue();
    }
    
    /** Efface et rinitialise tous les composants du panel */ 
    public void cleanAll() {
        m_itemsetsList.setListData(new Vector());
        m_supportSlider.setValue(50);
        m_log.clean();
    }
}