/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 *  MerjaFrame.java
 *  2003-2004 Darnet Camille, Vouriot Thierry
 *  camsss@ifrance.com, yeri@fr.st
 *  http://www.yeri.fr.st
 *
 */
package merja.gui;

import java.awt.event.*;
import java.awt.*;
import java.io.*;
import java.util.Date;
import java.text.SimpleDateFormat;
import javax.swing.*;
import merja.*;
import merja.data.*;
import merja.gui.actions.MerjaAction;
import merja.images.Icons;
import merja.associations.*;

/** Fentre principale de Merja
 */
public class MerjaFrame extends JFrame {
    
    private final Chrono m_chrono = new Chrono(); 
    private TaskMonitor m_taskMonitor;
    private LogPanel m_logPanel;
    private MerjaToolBar m_toolBar;
    private MerjaTabbedPane m_tabbedPane;
    
    /** Constructeur
     */
    public MerjaFrame() {
        Merja.initUI();
        Utilities.changePlaf(this,"metal");
        Merja.initActions();
        MerjaAction.setFrame(this);
        
        this.setTitle("Merja");
        this.setIconImage(this.getToolkit().createImage(Utilities.getURL(Icons.ICON_MERJA,Merja.class)));
        this.setSize(640, 480);
        this.setLocation(Utilities.getCenterComponent(this));
        
        this.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
                System.exit(0);
            }
        });
        m_taskMonitor = new TaskMonitor();
        m_logPanel = new LogPanel(m_taskMonitor);
        
        Container contentPane = this.getContentPane();
        m_logPanel.logMessage(System.getProperty("os.name") + " " + System.getProperty("os.version") + " " + System.getProperty("os.arch"));
        m_logPanel.logMessage("Java " + System.getProperty("java.version") + " " + System.getProperty("java.vendor"));
        m_logPanel.logMessage("Merjadir : " + Merja.getMerjaHome());
        String date = (new SimpleDateFormat("EEEE, d MMMM yyyy")).format(new Date());
        m_logPanel.logMessage("Dmarr le " + date);
        
        m_logPanel.statusMessage("Merja v" + Merja.RELEASE + " (build " + Merja.BUILD + ")");
        m_tabbedPane = new MerjaTabbedPane(m_logPanel, this);
        activePanelsAndButtons(true, false, false, false);
        
        m_toolBar = new MerjaToolBar();
        
        contentPane.setLayout(new BorderLayout());
        contentPane.add(m_tabbedPane,BorderLayout.CENTER);
        contentPane.add(m_toolBar,BorderLayout.WEST);
        contentPane.add(m_logPanel,BorderLayout.SOUTH);
    }
    
    /** Retourne le panel de log
     * @return panel de log
     */    
    public LogPanel getLogPanel() {
        return m_logPanel;
    }
    
    /** Retourne le <CODE>DataPanel</CODE>
     * @return le <CODE>DataPanel</CODE>
     */    
    public DataPanel getDataPanel() {
        return this.m_tabbedPane.m_dataPanel;
    }
    
    /** Retourne le <CODE>FileDBPanel</CODE>
     * @return le <CODE>FileDBPanel</CODE>
     */    
    public FileDBPanel getFileDBPanel() {
        return this.m_tabbedPane.m_fileDBPanel;
    }
    
    /** Retourne le <CODE>ItemsetsPanel</CODE>
     * @return le <CODE>ItemsetsPanel</CODE>
     */    
    public ItemsetsPanel getItemsetsPanel() {
        return this.m_tabbedPane.m_itemsetsPanel;
    }
    
    /** Retourne le <CODE>AssociationRulesPanel</CODE>
     * @return le <CODE>AssociationRulesPanel</CODE>
     */    
    public AssociationRulesPanel getAssociationRulesPanel() {
        return this.m_tabbedPane.m_associationRulesPanel;
    }
    
    /** Selectionne un panel
     * @param panel panel  slectionner
     */    
    public void setTabPanel(MerjaPanel panel) {
        this.m_tabbedPane.setSelectedComponent(panel);
    }
    
    /** Retourne le <CODE>TransactionsReader</CODE>
     * @return le <CODE>TransactionsReader</CODE>
     */    
    public TransactionsReader getTR() {
        return this.m_tabbedPane.m_fileDBPanel.getTR();
    }
    
    /** Retourne le <CODE>AlgoInterface</CODE>
     * @return le <CODE>AlgoInterface</CODE>
     */    
    public AlgoInterface getAlgoInterface() {
        return this.m_tabbedPane.m_fileDBPanel.getAlgoInterface();
    }
    
    /** Change le <CODE>AlgoInterface</CODE>
     * @param af nouveau <CODE>AlgoInterface</CODE>
     */    
    public void setAlgoInterface(AlgoInterface af) {
        this.m_tabbedPane.m_fileDBPanel.setAlgoInterface(af);
    }
    
    /** Retourn le chronomtre
     * @return chronomtre
     */    
    public Chrono getChrono() {
        return m_chrono;
    }
    
    /** Retourne le panel courant
     * @return panel courant
     */    
    public MerjaPanel getCurrentPanel() {
        return (MerjaPanel)(this.m_tabbedPane.getSelectedComponent());
    }
    
    /** Active ou dsactive certains onglets et boutons
     * @param t1 onglet File/BD
     * @param t2 onglet Data
     * @param t3 onglet Itemsets frquents
     * @param t4 onglet Rgles d'association
     */    
    public void activePanelsAndButtons(boolean t1, boolean t2, boolean t3, boolean t4) {
        m_tabbedPane.setEnabledAt(0, t1);
        m_tabbedPane.setEnabledAt(1, t2);
        m_tabbedPane.setEnabledAt(2, t3);
        m_tabbedPane.setEnabledAt(3, t4);
        
        m_toolBar.m_assocRulesBtn.setEnabled(t4);
        m_toolBar.m_itemsetsBtn.setEnabled(t3);
    }
}
