/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 *  IdItemSupport.java
 *  2003-2004 Darnet Camille, Vouriot Thierry
 *  camsss@fr.st, yeri@fr.st
 *  http://www.yeri.fr.st
 *
 */
package merja.associations.itemFreq;

public class IdItemSupport {
    private int itemId;
    private int support;
    private NoeudItem suivant;
    
    /**
     * Reprsente un item comme lment d'un noeud de l'arbre des ensembles frquents
     * @param itemId : identifiant d'un items
     */
    public IdItemSupport( int itemId){
        this.itemId=itemId;
        GeneItems.nbIdItemSupport++;
        suivant=null;
        support=0;
    }
    
    /**
     *
     * @return Identifiant de l'item
     */
    public int getItemId(){
        return itemId;
    }
    
    /**
     *
     * @return true si il a suivant dans l'arbre
     * <br> false sinon
     */
    public boolean asSuivant(){
        if (suivant==null) return false;
        return true;
    }
    
    /**
     *
     * @param noeud : le suivant dans l'arbre des ensembles frquents
     */
    public void setSuivant(NoeudItem noeud){
        this.suivant=noeud;
    }
    
    /**
     *
     * @return le suivant dans l'arbre des ensembles frquents
     */
    public NoeudItem getSuivant(){
        return suivant;
    }
    
    /**
     * Renvoie le support de l'ensemble frquent exprim par l'arbre et  ayant comme
     * dernier lment cet item
     * @return un entier
     */
    public int getSupport(){
        return this.support;
    }
    
    /**
     * Incrmentation du support
     */
    public void addSupport(){
        this.support++;
    }
    
    /**
     * Test la validit du support
     * @return : true si le support est infrieur  GeneItems.supportMin
     *           false sinon
     */
    public boolean infSupportMin(){
        if (support< GeneItems.supportMin ) return true;
        return false;
    }
    
    /**
     * Test la validit du support
     * @return : true si le support est suprieur  GeneItems.supportMin
     *           false sinon
     */
    public boolean isValideSupport(){
        if (support >= GeneItems.supportMin) return true;
        return false;
    }
    
}