/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 *  NoeudItem.java
 *  2003-2004 Darnet Camille, Vouriot Thierry
 *  camsss@fr.st, yeri@fr.st
 *  http://www.yeri.fr.st
 *
 */
package merja.associations.itemFreq;

import java.util.Vector;

public class NoeudItem {
    private IdItemSupport[] idItemSupports;
    
    /**
     * Constructeur d'un noeud d'items
     * @param idItems :  liste d'items trie
     */
    public NoeudItem(int[] idItems) {
        this(idItems.length);
        //idItemSupports = new IdItemSupport[idItems.length];
        // construction de la liste des items
        for (int i = 0; i < idItemSupports.length; i++) {
            idItemSupports[i] = new IdItemSupport(idItems[i]);
        }
        // free(idItems)
    }
    
    /**
     * Constructeur d'un noeud d'items
     * @param length : nombre d'item qui sera ajout
     */
    public NoeudItem(int length) {
        GeneItems.nbNoeud++;
        //  System.out.println(GeneItems.nbNoeud);
        idItemSupports = new IdItemSupport[length];
        for (int i = 0; i < idItemSupports.length; i++)
            idItemSupports[i] = null;
    }
    
    /**
     * Ajout d'identifiant dans le noeud
     * <br> attention l'identifiant doit tre suprieur aux identifiant dja ajouts
     * @param itemId
     */
    public void add(int itemId) {
        int i = 0;
        while (idItemSupports[i] != null)
            i++;
        idItemSupports[i] = new IdItemSupport(itemId);
    }
    
    /**
     * Supprimes les ensembles candidats d'un niveau ( = hauteur de l'arbre)
     * @param niveau
     * @return true si tous les lments sont supprims
     * <br> false sinon
     */
    protected boolean removes(int niveau) {
        boolean resu = false;
        // niveau o il y a les supports candidats
        if (niveau == GeneItems.hauteur) {
            //sauvegarde des idItemSupports valides
            Vector valideSupport = new Vector();
            //parcours de tous les idItemSupports
            for (int i = 0; i < idItemSupports.length; i++) {
                // test  de validit du support
                if (idItemSupports[i].getSupport() >= GeneItems.supportMin) {
                    // test ok alors sauvegarde
                    valideSupport.add(idItemSupports[i]);
                    resu = true;
                }
                else {
                    // suppression car support invalide
                    idItemSupports[i] = null;
                    GeneItems.nbIdItemSupport--;
                }
            }
            //Raffectation des items ayant des support valides
            idItemSupports = new IdItemSupport[valideSupport.size()];
            for (int i = 0; i < idItemSupports.length; i++) {
                idItemSupports[i] = (IdItemSupport) valideSupport.get(i);
            }
        }
        // descente dans l'arbre
        else {
            boolean test;
            boolean notSuivant = true;
            // parcours de tous les IditemSupport
            for (int i = 0; i < idItemSupports.length; i++) {
                // si il a des suivants
                if (idItemSupports[i].asSuivant()) {
                    notSuivant = false;
                    // rcurence sur le noeud suivant
                    test = idItemSupports[i].getSuivant().removes(niveau + 1);
                    if (!test) {
                        // si tout les lments de suivant sont supprims
                        // alors suppression de noeud suivant
                        idItemSupports[i].setSuivant(null);
                        GeneItems.nbNoeud--;
                    }
                    else
                        resu = true;
                }
            }
            if (notSuivant)
                resu = true;
        }
        return resu;
    }
    
    /**
     * Calcul les supports des ensembles candidats  partir de la transaction exprime
     * sous forme de tableau d'identifiant
     * @param itemsId
     * @param indiceCourant
     */
    protected void calculSupport(int[] itemsId, int indiceCourant) {
        //niveau o il faut calculer le nouveau support
        if (indiceCourant == GeneItems.hauteur) {
            int indice;
            //calculer les support pour les items de itemsID existant dans le noeud
            for (int i = 0; i < itemsId.length; i++) {
                // recherche de l'indice de l'item courant
                indice = getItemPos(itemsId[i]);
                // si item existe dans le noeud
                if (indice != -1) {
                    // incrmentation du support
                    //System.out.println(GeneItems.hash.getItem(idItemSupports[indice].getItemId()));
                    idItemSupports[indice].addSupport();
                }
            }
        }
        // sinon continuer  descendre dans l'arbre
        else {
            // !!!!!!!!!!!! tableau suivant decalage
            int[] tabNew = new int[itemsId.length - 1];
            // l'lment courant dans la liste des itemsId existe dans le noeud
            int indice, i, j, k;
            for (i = 0; i < itemsId.length; i++) {
                indice = getItemPos(itemsId[i]);
                if (indice != -1) {
                    if (idItemSupports[indice].asSuivant()) {
                        for (j = 0; j < i; j++) {
                            tabNew[j] = itemsId[j];
                        }
                        for (k = j + 1; k < itemsId.length; k++) {
                            tabNew[j] = itemsId[k];
                            j++;
                        }
                        idItemSupports[indice].getSuivant().calculSupport(tabNew,
                        indiceCourant + 1);
                    }
                }
            }
            
        }
    }
    
    /**
     * test de validit d'un ensemble d'items
     * @param idItemList : ensemble d'items exprim sous forme d'une liste trie
     *  d'ientifiant
     * @param niveau
     * @return true si l'ensemble est frquent
     * <br> false sinon
     */
    protected boolean isValide(int[] idItemList, int niveau) {
        boolean resu = false;
        //arrive au bout de la liste id  tester
        if (niveau == idItemList.length - 1) {
            int indice;
            // recuperation de l'indice
            indice = getItemPos(idItemList[niveau]);
            // si cet indice existe
            if (indice != -1)
                
                // test si le support est valide
                resu = idItemSupports[indice].isValideSupport();
        }
        // recurence jusqu' la fin de la liste
        else {
            int indice;
            // recuperation de l'indice
            indice = getItemPos(idItemList[niveau]);
            // si cet indice existe
            if (indice != -1) {
                // si un suivant existe
                if (idItemSupports[indice].asSuivant())
                    
                    // recurence sur le niveau
                    resu = idItemSupports[indice].getSuivant().isValide(idItemList,
                    niveau + 1);
            }
        }
        
        return resu;
    }
    
    /**
     * Permet la gnration des ensembles d'items candidats  partir des ensembles
     * frquents
     * @param niveau :  niveau des ensembles frquents
     * @param idItemList : liste d'items qui correspond  l'ensemble frquent jusqu'au
     * noeud courant
     */
    public void jointure(int niveau, int[] idItemList) {
        //Niveau o la jointure doit tre executer
        if (niveau == GeneItems.hauteur) {
            Vector tabTmp = new Vector();
            int i, j, k;
            int tabLength;
            NoeudItem newNoeud;
            // pour tous les ss-noeuds
            for (i = 0; i < idItemSupports.length; i++) {
                tabTmp.removeAllElements();
                tabLength = 0;
                // creer les joinutes avec les suivants
                if (idItemSupports[i].isValideSupport()) {
                    if (niveau > 0)
                        idItemList[niveau - 1] = idItemSupports[i].getItemId();
                    
                    for (j = i + 1; j < idItemSupports.length; j++) {
                        // test si la sous chaine existes
                        idItemList[niveau] = idItemSupports[j].getItemId();
                        
                        for (k = 0; k < niveau + 1; k++)
                            
                            if (GeneItems.noeud0.isValide(idItemList, 0)) {
                                tabTmp.add(new Integer(idItemSupports[j].getItemId()));
                                tabLength++;
                            }
                    }
                    // creation du nouveau noeud
                    if (tabLength > 0) {
                        newNoeud = new NoeudItem(tabLength);
                        for (k = 0; k < tabLength; k++) {
                            newNoeud.add( ( (Integer) tabTmp.get(k)).intValue());
                        }
                        // affectation du nouveau noeud
                        idItemSupports[i].setSuivant(newNoeud);
                    }
                }
            }
        }
        // recurennce sur le niveau pour atteindre la hauteur
        else {
            
            // recurence sur tous les lments de idItemSupports
            for (int i = 0; i < idItemSupports.length; i++) {
                // si ils ont des suivants
                if (idItemSupports[i].asSuivant()) {
                    if (niveau != 0)
                        idItemList[niveau - 1] = idItemSupports[i].getItemId();
                    idItemSupports[i].getSuivant().jointure(niveau + 1, idItemList);
                }
            }
        }
    }
    
    /**
     *
     * @param idItem
     * @return indice si idItem existe dans la liste IdItemSupport
     * Sinon -1
     * Recherche dichotomique
     */
    private int getItemPos(int idItem) {
        int indiceMil, indiceMin, indiceMax;
        indiceMin = 0;
        indiceMax = idItemSupports.length;
        indiceMil = (indiceMin + indiceMax) / 2;
        while (indiceMin != indiceMil) {
            if (idItemSupports[indiceMil].getItemId() <= idItem)
                indiceMin = indiceMil;
            else
                indiceMax = indiceMil;
            indiceMil = (indiceMin + indiceMax) / 2;
        }
        if (idItemSupports[indiceMil].getItemId() != idItem)
            return -1;
        return indiceMil;
    }
    
    public String toString(int niveau, String debut) {
        StringBuffer tmp = new StringBuffer("");
        String courant;
        int k;
        for (int i = 0; i < idItemSupports.length; i++) {
            if (idItemSupports[i].isValideSupport()) {
                courant = " | " +
                GeneItems.hash.getItem(idItemSupports[i].getItemId()).toString();
                if (idItemSupports[i].asSuivant()) {
                    tmp.append(idItemSupports[i].getSuivant().toString(niveau + 1,
                    debut + courant));
                }
                tmp.append(debut);
                tmp.append(courant);
                tmp.append(" => ");
                tmp.append(idItemSupports[i].getSupport());
                tmp.append("\n");
            }
        }
        return tmp.toString();
    }
    
    /**
     *
     * @param listeItems
     * @param precedant
     */
    protected void addItemSet(Vector listeItems, ItemSet precedant) {
        for (int i = 0; i < idItemSupports.length; i++) {
            if (idItemSupports[i].isValideSupport()) {
                ItemSet tmp = new ItemSet(precedant,
                GeneItems.hash.getItem(idItemSupports[i].
                getItemId())
                );
                tmp.setSupport(idItemSupports[i].getSupport());
                listeItems.add(tmp);
                if (idItemSupports[i].asSuivant()) {
                    idItemSupports[i].getSuivant().addItemSet(listeItems, tmp);
                }
            }
        }
        
    }
    
    /**
     * Ajoute les items sous la forme de string dans la vector v
     * @param v = Vector de stockage
     * @param niveau = niveau dans l'arbre
     * @param debut = dbut de l'item frquent
     */
    public void addItem(Vector v, int niveau, String debut) {
        StringBuffer tmp;
        String courant;
        int k;
        for (int i = 0; i < idItemSupports.length; i++) {
            if (idItemSupports[i].isValideSupport()) {
                courant = " | " +
                GeneItems.hash.getItem(idItemSupports[i].getItemId()).toString();
                if (idItemSupports[i].asSuivant()) {
                    idItemSupports[i].getSuivant().addItem(v, niveau + 1, debut + courant);
                }
                tmp = new StringBuffer("");
                tmp.append(debut);
                tmp.append(courant);
                tmp.append(" => ");
                tmp.append(idItemSupports[i].getSupport());
                
                v.add(tmp.toString());
            }
        }
        
    }
    
    /**
     * Permet la rcupration de tous les ensembles d'items
     * @param itemsFreq : Sauvegarde des ensembles
     * @param listeItems  :liste d'items qui correspond  l'ensemble frquent jusqu'au
     * noeud courant
     * @param indiceCourant : indice du dernier lment dans la liste d'items
     */
    public void addItemsFrequent(Vector itemsFreq, int[] listeItems,
    int indiceCourant) {
        for (int i = 0; i < idItemSupports.length; i++) {
            listeItems[indiceCourant] = idItemSupports[i].getItemId();
            if (idItemSupports[i].asSuivant()) {
                idItemSupports[i].getSuivant().addItemsFrequent(itemsFreq, listeItems,
                indiceCourant + 1);
            }
            int[] listeSave = new int[indiceCourant + 1];
            for (int j = 0; j < listeSave.length; j++) {
                listeSave[j] = listeItems[j];
            }
            itemsFreq.add(listeSave);
        }
    }
    
    /**
     * Permet de retrouver le support d'un ensemble frquents
     * <br> ATTENTION il faut que l'ensemble d'items frquents soit dans l'arbre
     * sinon a PLANTE !!!!!!
     * @param listeItems corresondant  un ensemble d'items frquents
     * @param indiceCourant indice de l'lment courant dans la liste d'items
     * @return le support
     *
     */
    public int getSupport(int[] listeItems, int indiceCourant) {
        int indice = this.getItemPos(listeItems[indiceCourant]);
        if ( (indiceCourant + 1) == listeItems.length) {
            return this.idItemSupports[indice].getSupport();
        }
        else {
            if (this.idItemSupports[indice].asSuivant())
                return this.idItemSupports[indice].getSuivant().getSupport(listeItems,
                indiceCourant + 1);
            else
                return -1;
        }
    }
    
}