/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 *  GeneRegles.java
 *  2003-2004 Darnet Camille, Vouriot Thierry
 *  camsss@fr.st, yeri@fr.st
 *  http://www.yeri.fr.st
 *
 */
package merja.associations.regles;

import java.util.Vector;
import merja.associations.itemFreq.GeneItems;
import merja.associations.Hash;
import merja.gui.Logger;

public class GeneRegles {
    private Vector itemFreq;
    private NoeudRegles[] regles;
    private int niveau;
    private GeneItems geneItems;
    private double minconf;
    
    /**
     * Table de correspodance
     * <br> remarque : utiliser avec prcautions
     */
    static protected Hash hash;
    /**
     * Donnes statistique sur le nombre de rgles
     * <br> remarque : peut -tre manipul n'importe comment a drange pas l'algo
     */
    static protected int nbRegles;
    /**
     * Donnes statistique sur le nombre de rgles valides
     * <br> remarque : peut -tre manipul n'importe comment a drange pas l'algo
     */
    static protected int nbReglesValide;
    
    /**
     * Cr une instance de GeneRegles
     * <br> cette classe permet de manipuler rgles
     * sans rentrer dans les dtails de la programmation
     * @param geneItems : arbre des items frquents
     * @param hash : table de correspondance entre les items et leurs identifiants
     * @param min : confiance minmimum
     */
    public GeneRegles(GeneItems geneItems, Hash hash, double min) {
        this.itemFreq = geneItems.getItemsFrequents();
        this.geneItems = geneItems;
        regles = new NoeudRegles[itemFreq.size()];
        niveau = 0;
        nbRegles = 0;
        nbReglesValide = 0;
        minconf = min;
        this.hash = hash;
    }
    
    /**
     * Gnration des rgles d'association
     * @param log
     */
    public void createRegle(Logger log) {
        int i;
        int[] itemsFrequents;
        NoeudRegles tmp;
        
        for (i = 0; i < regles.length; i++) {
            log.logMessage("Itration : " + i);
            itemsFrequents = (int[]) (itemFreq.get(i));
            tmp = new NoeudRegles(itemsFrequents, geneItems,
            geneItems.getSupport(itemsFrequents), this.minconf);
            log.logMessage("Nb de rgles  : " + nbRegles);
            regles[i] = tmp;
        }
    }
    
    /**
     *
     * @return les rgles valides
     */
    public String toString() {
        int i;
        StringBuffer resu = new StringBuffer("");
        nbReglesValide = 0;
        for (i = 0; i < regles.length; i++) {
            resu.append(regles[i].toString(hash));
        }
        return resu.toString();
    }
    
    /**
     * Obsoleste
     * Retourne les rgles valides sous forme de string dans un vector
     * @return les rgles valides dans un Vector
     */
    public Vector getRegleValideString() {
        Vector resu = new Vector();
        int i;
        for (i = 0; i < regles.length; i++) {
            regles[i].addRegleValide(resu);
        }
        return resu;
    }
    /**
     * Retourne toutes les regles sous la forme de RegleSet
     */
    public Vector getRegleSet(){
        Vector resu = new Vector();
        int i;
        for(i=0;i<regles.length;i++){
            regles[i].addRegleSet(resu,hash);
        }
        return resu;
    }
    
    /**
     * Permet de changer la confiance minimum
     * @param min
     */
    public void setMinConf(double min) {
        nbReglesValide = 0;
        this.minconf = min;
        for (int i = 0; i < regles.length; i++)
            regles[i].setMinConf(min);
    }
    
    public Hash getHashTables(){
        return this.hash;
    }
    
    /**
     *
     * @return le nombre de rgles
     */
    public int getNbRegles() {
        return this.nbRegles;
    }
    
    /**
     *
     * @return le nombre de rgles valides
     */
    public int getNbReglesValides() {
        return this.nbReglesValide;
    }
    
}