/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 *  RegleSet.java
 *  2003-2004 Darnet Camille, Vouriot Thierry
 *  camsss@fr.st, yeri@fr.st
 *  http://www.yeri.fr.st
 *
 */

package merja.associations.regles;

import merja.associations.Hash;
import merja.associations.Item;

/** Represente une rgle d'association */
public class RegleSet {
    private Item[] first;
    private int supportFirst;
    private Item[] second;
    private int supportSecond;
    private double confiance;
    
    /** Constructeur
     * @param fi tableau d'items correspondant  la partie gauche
     * @param suFi support de la partie gauche
     * @param se tableau d'items correspondant  la partie droite
     * @param suSe support de la partie droite
     * @param conf confiance de la rgle
     * @param h table de hashage
     */    
    public RegleSet(int[] fi,int suFi, int[] se,int suSe,double conf, Hash h) {
        first = new Item[fi.length];
        second = new Item[se.length];
        this.supportFirst=suFi;
        this.supportSecond=suSe;
        this.confiance=conf;
        int i;
        for (i = 0; i < first.length; i++) {
            first[i] = h.getItem(fi[i]);
        }
        for (i = 0; i < second.length; i++) {
            second[i] = h.getItem(se[i]);
        }
    }
    
    /** Retourne un tableau d'items correspondant  la partie gauche de la rgle
     * @return tableau d'items
     */    
    public Item[] getFirstItems(){
        return this.first;
    }
    
    /** Retourne un tableau d'items correspondant  la partie droite de la rgle
     * @return tableau d'items
     */    
    public Item[] getSecondItems(){
        return this.second;
    }
    
    /** Retourne un tableau contenant tous les items de la rgle
     * @return tableau d'items
     */    
    public Item[] getAllItems(){
        Item[] all = new Item[first.length+second.length];
        int i;
        for(i=0;i<first.length;i++)
            all[i]=first[i];
        for(int j=0;j<second.length;j++)
            all[i+j]=second[j];
        return all;
    }
    
    
    /** Retourne une chane permettant l'affichage d'une rgle
     * @return chane de caractre
     */    
    public String toString(){
        StringBuffer resu= new StringBuffer("");
        int i;
        for (i = 0; i < first.length - 1; i++)
            resu.append(first[i].toString() + " , ");
        resu.append(first[i].toString());
        resu.append(" ("+supportFirst +") ");
        resu.append(" => ");
        for (i = 0; i < second.length - 1; i++)
            resu.append(second[i].toString() + " , ");
        resu.append(second[i].toString() + " : ");
        resu.append(" ("+supportSecond+") ");
        resu.append(" " + ( ( ( (double) ( (int) (confiance * 1000))) / 1000)));
        
        return resu.toString();
        
    }
    
}