/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 *  CvsLoader.java
 *  2003-2004 Darnet Camille, Vouriot Thierry
 *  camsss@ifrance.com, yeri@fr.st
 *  http://www.yeri.fr.st
 *
 */
package merja.data;

import java.util.*;
import java.io.*;
import merja.associations.*;

/** Chargeur de fichier au format <CODE>csv</CODE> */
public class CsvLoader implements TransactionsReader {
    
    private String m_name;
    private RandomAccessFile m_accessFile;
    private Hashtable m_itemsTable = new Hashtable();
    private int m_nbTransactions = 0;
    private int m_nbItems;
    
    /** Constructeur
     * @param filePath chemin d'accs du fichier
     * @throws IOException erreur lors du chargement du fichier
     */    
    public CsvLoader(String filePath) throws IOException {
        m_nbItems = 0;
        m_accessFile = new RandomAccessFile(filePath, "r");
        m_name = new File(filePath).getName();
        String line;
        StringTokenizer token;
        line = m_accessFile.readLine();
        if (line == null) throw new IOException("Csv format invalide"); // la premiere ligne contient (Transactionid;Items)
        while ((line = m_accessFile.readLine()) != null) {
            token = new StringTokenizer(line, ";");
            String tmp;
            if (token.hasMoreTokens()) {
                tmp = token.nextToken();    // transactionid
                while (token.hasMoreTokens()) {
                    tmp = token.nextToken();
                    if (!m_itemsTable.containsKey(tmp)) m_itemsTable.put(tmp, "");
                    m_nbItems++;
                }
                m_nbTransactions++;
            }
        }
        goToFirstTransaction();
    }
    
    /** Retourne l'ensemble des attributs avec leurs valeurs
     * @return tableau d'items correspondant  tous les items diffrents
     */    
    public Item[] getAttributes() {
        Enumeration enum = m_itemsTable.keys();
        Vector itemsVector = new Vector();
        while (enum.hasMoreElements())
            itemsVector.add(new Item("",(String)enum.nextElement()));
        
        Object[] tab = itemsVector.toArray();
        Item[] items = new Item[tab.length];
        for (int i=0; i<tab.length; i++)
            items[i] = (Item)tab[i];
        return items;
    }
    
    /** Place le pointeur sur la premire transaction de la base de donnes */    
    public void goToFirstTransaction() {
        try {
            m_accessFile.seek(0);
            m_accessFile.readLine();
        }
        catch (IOException ioe) { System.out.println("CvsLoader::goToFirstTransaction : "+ioe); }
    }
    
    /** Retourne la prochaine transaction (tableau d'items)
     * @return pochaine transaction (tableau d'items)
     */   
    public Item[] nextTransaction() {
        String line = null;
        try {
            line = m_accessFile.readLine();
        }
        catch (IOException ioe) { System.out.println("CvsLoader::nextTransaction : "+ioe); }
        if (line == null) return null;
        
        Vector itemsVector = new Vector();
        StringTokenizer token = new StringTokenizer(line, ";");
        while (token.hasMoreTokens())
            itemsVector.add(new Item("",token.nextToken()));
        
        Object[] tab = itemsVector.toArray();
        if (tab.length == 0) return null;
        Item[] items = new Item[tab.length-1];
        for (int i=1; i<tab.length; i++)
            items[i-1] = (Item)tab[i];
        return items;
    }
    
    /** Retourne le nombre d'attributs
     * @return nombre d'attributs (pas d'attributs pour les fichiers csv)
     */    
    public int getNbAttributes() {
        return 0;
    }
    
    /** Retourne le nom du fichier ou de la base de donnes
     * @return nom de la base de donnes
     */    
    public String getRelationName() {
        return m_name;
    }
    
    /** Main pour tester la classe
     * @param argv arguments de la ligne de commande
     */    
    public static void main(String []argv) {
        String filePath = "d:/toto.csv";
        TransactionsReader fr = null;
        try {
            fr = new CsvLoader(filePath);
        }
        catch (Exception e) {System.out.println("TOTO" + e.toString());}
        Item[] item;
        while ((item = fr.nextTransaction()) != null) {
            for (int i=0; i <item.length; i++)
                System.out.print(item[i].toString()+";");
            System.out.println();
        }
    }
    
    /** Retourne le nombre de transactions de la base de donnes
     * @return nombre de transactions total
     */    
    public int getNbTransactions() {
        return m_nbTransactions;
    }
    
    /** Retourne le nombre d'items total
     * @return nombre d'items total
     */    
    public int getNbItems() {
        return m_nbItems;
    }
    
    /** Retourne le nombre d'items ayant des valeurs diffrentes
     * @return nombre d'items diffrents
     */    
    public int getNbItemsDiff() {
        return getAttributes().length;
    }
}

