/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 *  DBLoader.java
 *  2003-2004 Darnet Camille, Vouriot Thierry
 *  camsss@ifrance.com, yeri@fr.st
 *  http://www.yeri.fr.st
 *
 */
package merja.data;

import java.util.*;
import java.io.*;
import java.sql.*;
import merja.db.*;
import merja.associations.*;

/** Classe permettant de fournir les mthodes de l'interface <CODE>TransactionsReader</CODE>
 * pour une base de donnes et une requte SQL
 */
public class DBLoader implements TransactionsReader {
    
    private final Vector m_itemsVector = new Vector();
    private Database m_db;
    private String m_table;
    private Vector m_attributesVector;
    private String m_name;
    private int m_nbAttributes = 0;
    private int m_nbTransactions = 0;
    private String m_query;
    
    /** Constructeur
     * @param db Base de donnes
     * @param sqlQuery requte SQL
     * @param name nom donn  la relation
     * @throws SQLException
     */
    public DBLoader(Database db, String sqlQuery, String name) throws SQLException {
        int i;
        m_db = db;
        m_query = sqlQuery;
        
        m_db.executeSQLQuery(m_query);
        
        m_attributesVector = new Vector();
        for (i=0; i<m_db.getResultSetMetaData().getColumnCount(); i++) {
            m_attributesVector.add(m_db.getResultSetMetaData().getColumnName(i+1)+"<"+m_db.getResultSetMetaData().getTableName(i+1));
        }
        
        // Compte le nombre de lignes
        while (m_db.getResultSet().next())
            m_nbTransactions++;
       
        m_nbAttributes = m_attributesVector.size();
        m_name = name;
        
        // Recherche tous les items diffrents
        for (i=0; i<m_attributesVector.size(); i++) {
            String[] attr = getAttributeTable(((String)m_attributesVector.get(i)));
            m_db.executeSQLQuery("SELECT DISTINCT(`"+attr[0]+"`) FROM `"+attr[1]+"`");
            
            while (m_db.getResultSet().next()) {
                m_itemsVector.addElement(new Item(((String)m_attributesVector.get(i)), m_db.getResultSet().getString(1)));     
            }
        }
        
        m_db.executeSQLQuery(m_query);
    }
    
    /** Prend une chaine de la forme attribut<nomdetable et retourne un tableau de 2
     * lments contenant l'attribut et le nom de la table
     * @param s chane  traiter
     * @return tableau de 2 <CODE>String</CODE>
     */
    public static String[] getAttributeTable(String s) {
        String res[] = new String[2];
        res[0] = s.substring(0,s.lastIndexOf('<'));
        res[1] = s.substring(s.lastIndexOf('<')+1);
        return res;
    }
    
    /** Retourne l'ensemble des attributs avec leurs valeurs
     * @return tableau d'items correspondant  tous les items diffrents
     */
    public Item[] getAttributes() {
        Object[] tab = m_itemsVector.toArray();
        Item[] items = new Item[tab.length];
        for (int i=0; i<tab.length; i++)
            items[i] = (Item)tab[i];
        return items;
    }
    
    /** Place le pointeur sur la premire transaction de la base de donnes */
    public void goToFirstTransaction() {
        m_db.executeSQLQueryNoEx(m_query);
    }
    
    /** Retourne la prochaine transaction (tableau d'items)
     * @return pochaine transaction (tableau d'items)
     */
    public Item[] nextTransaction() {
        Item [] tab = new Item [m_nbAttributes];
        try {
            if (!m_db.getResultSet().next()) return null;
            
            for (int i=0; i<m_nbAttributes; i++) {
                tab[i] = new Item((String)m_attributesVector.get(i), m_db.getResultSet().getString(i+1));
            }
        }
        catch (SQLException sqle) { System.out.println("DBLoader::nextTransaction : "+sqle); }
        
        return tab;
    }
    
    /** Retourne le nombre d'attributs
     * @return nombre d'attributs
     */
    public int getNbAttributes() {
        return m_nbAttributes;
    }
    
    /** Retourne le nom du fichier ou de la base de donnes
     * @return nom de la base de donnes
     */
    public String getRelationName() {
        return m_name;
    }
    
    /** Retourne le nombre de transactions de la base de donnes
     * @return nombre de transactions total
     */
    public int getNbTransactions() {
        return m_nbTransactions;
    }
    
    /** Retourne le nombre d'items total
     * @return nombre d'items total
     */
    public int getNbItems() {
        return getNbTransactions()*getNbAttributes();
    }
    
    /** Retourne le nombre d'items ayant des valeurs diffrentes
     * @return nombre d'items diffrents
     */
    public int getNbItemsDiff() {
        return getAttributes().length;
    }
    
    /** Retourne la requte SQL correspondant aux donnes traites
     * @return requte SQL
     */
    public String getSQLQuery() {
        return this.m_query;
    }
}

