/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 *  AssociationRulesPanel.java
 *  2003-2004 Darnet Camille, Vouriot Thierry
 *  camsss@ifrance.com, yeri@fr.st
 *  http://www.yeri.fr.st
 *
 */
package merja.gui;

import java.util.*;
import java.awt.event.*;
import java.awt.*;
import javax.swing.*;
import javax.swing.event.*;
import merja.*;
import merja.images.Icons;
import merja.associations.*;
import merja.associations.regles.*;

/** Panel des rgles d'association */
public class AssociationRulesPanel extends MerjaPanel {
    
    private final JList m_rulesList = new JList();
    private final JSlider m_confianceSlider = new JSlider(JSlider.HORIZONTAL,1,100,50);
    private final JButton m_startBtn = new JButton("Extraire les rgles");
    private final JButton m_stopBtn = new JButton("Stop");
    private final LogTextArea m_log = new LogTextArea();
    
    /** Constructeur */
    AssociationRulesPanel() {
        this.setLayout(new BorderLayout());
        
        JPanel panelWest = new JPanel(new BorderLayout());
        JPanel panelCenter = new JPanel(new BorderLayout());
        
        //m_rulesList.setEditable(false);
        m_stopBtn.setEnabled(false);
        
        panelCenter.add(new JScrollPane(m_rulesList),BorderLayout.CENTER);
        panelCenter.setBorder(BorderFactory.createCompoundBorder(
        BorderFactory.createTitledBorder("Rgles sortie"),
        BorderFactory.createEmptyBorder(0, 0, 0, 0)
        ));
        
        final JLabel confianceLabel = new JLabel("50%");
        JPanel panelConfiance = new JPanel(new BorderLayout());
        m_confianceSlider.setMinorTickSpacing(5);
        m_confianceSlider.setMajorTickSpacing(10);
        m_confianceSlider.setPaintTicks(true);
        m_confianceSlider.setPaintTrack(true);
        m_confianceSlider.setPreferredSize(new Dimension(110, 30));
        confianceLabel.setPreferredSize(new Dimension(30, 15));
        panelConfiance.add(m_confianceSlider,BorderLayout.CENTER);
        panelConfiance.add(confianceLabel,BorderLayout.EAST);
        panelConfiance.setBorder(BorderFactory.createCompoundBorder(
        BorderFactory.createTitledBorder("Confiance minimum"),
        BorderFactory.createEmptyBorder(0, 0, 0, 0)
        ));
        
        JPanel panelStartStop = new JPanel(new GridLayout(2,1,4,2));
        panelStartStop.add(m_startBtn);
        panelStartStop.add(m_stopBtn);
        panelStartStop.setBorder(BorderFactory.createEmptyBorder(3, 3, 3, 1));
        
        panelWest.add(panelConfiance,BorderLayout.NORTH);
        panelWest.add(m_log,BorderLayout.CENTER);
        panelWest.add(panelStartStop,BorderLayout.SOUTH);
        
        this.add(panelWest,BorderLayout.WEST);
        this.add(panelCenter,BorderLayout.CENTER);
        
        m_confianceSlider.addChangeListener(new ChangeListener() {
            public void stateChanged(ChangeEvent evt) {
                String s = m_confianceSlider.getValue()+"%";
                confianceLabel.setText(s);
            }
        });
        
        m_startBtn.addActionListener(new ActionListener() {
            public void actionPerformed(final java.awt.event.ActionEvent evt) {
                calcAssocRules();
            }
        });
        
        m_stopBtn.addActionListener(new ActionListener() {
            public void actionPerformed(final java.awt.event.ActionEvent evt) {
                stopThread();
                enableDisableButtons();
            }
        });
        
        m_rulesList.addMouseListener(new MouseAdapter() {
            public void mouseClicked(final java.awt.event.MouseEvent evt) {
                if (evt.getClickCount() == 2 && m_merjaFrame.getFileDBPanel().getDataType()==FileDBPanel.DB) {  
                    Point p = new Point(evt.getX(), evt.getY());
                    SwingUtilities.convertPointToScreen(p,  evt.getComponent());
                    RegleSet is = (RegleSet)(m_rulesList.getModel().getElementAt(m_rulesList.locationToIndex(evt.getPoint())));
                    Item[] items = is.getAllItems();
                    new DBItemsetFrame(m_merjaFrame, items, is.toString(), p).show();
                }
            }
        });
    }
    
    /** Active ou desactive les boutons <I>start</I> et <I>stop</I> suivant qu'il y ai
     * un traitement en cours ou non
     */
    public void enableDisableButtons() {
        m_stopBtn.setEnabled(m_runThread != null);
        m_startBtn.setEnabled(m_runThread == null);
    }
    
    /** Calcule et affiche les rgles d'association */
    public void calcAssocRules() {
        if (m_runThread == null) {
            m_runThread = new Thread("<association_rules>") {
                public void run() {
                    enableDisableButtons();
                    m_log.clean();
                    m_merjaFrame.getAlgoInterface().setLog(m_log);
                    m_logPanel.taskStarted();
                    m_logPanel.logMessage("DEBUT : Calcul des rgles d'association");
                    m_rulesList.setListData(new Vector());
                    m_merjaFrame.getChrono().start();
                    m_merjaFrame.getAlgoInterface().setConfianceMin(getConfiance());
                    m_merjaFrame.getAlgoInterface().geneRegle();
                    m_merjaFrame.getChrono().stop();
                    m_merjaFrame.getAssociationRulesPanel().setRules(m_merjaFrame.getAlgoInterface().getRegleSet());
                    m_logPanel.logMessage("FIN : Calcul des rgles d'association (" + m_merjaFrame.getChrono().timeElapsed() + "ms) : confiance = " + getConfiance());
                    m_logPanel.taskFinished();
                    m_runThread = null;
                    enableDisableButtons();
                }
            };
            m_runThread.setPriority(Thread.MIN_PRIORITY);
            m_runThread.start();
        }
    }
    
    private void setRules(Vector v) {
        this.m_rulesList.setListData(v);
    }
    
    private double getConfiance() {
        return (double)(this.m_confianceSlider.getValue())/100.0;
    }
    
    /** Efface et rinitialise tous les composants du panel */
    public void cleanAll() {
        m_rulesList.setListData(new Vector());
        m_confianceSlider.setValue(50);
        m_log.clean();
    }
}
