/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 *  DBItemsetFrame.java
 *  2003-2004 Darnet Camille, Vouriot Thierry
 *  camsss@fr.st, yeri@fr.st
 *  http://www.yeri.fr.st
 *
 */
package merja.gui;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.table.*;
import merja.*;
import merja.data.*;
import merja.db.*;
import merja.images.*;
import merja.gui.*;
import merja.associations.*;

/** Fentre permettant l'affichage des lignes d'un <CODE>ResultSet</CODE>
 * correspondant  un itemset frquent ou  une rgle d'association
 */
public class DBItemsetFrame extends JFrame {
    
    /** Constructeur
     * @param items Items de l'itemset frquents ou de la rgle d'association
     * @param title Titre de la fentre
     * @param p Position de la fentre
     * @param fr fentre principale de l'application Merja
     */
    public DBItemsetFrame(final MerjaFrame fr, final Item[] items, String title, Point p) {
        
        this.setTitle(title);
        this.setSize(400,260);
        this.setLocation(p);
        this.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        this.setIconImage(Utilities.getImage(Icons.ICON_NEXT, Merja.class));
        
        Container contentPane = this.getContentPane();
        contentPane.setLayout(new BorderLayout());
        
        final JLabel lblStatus = new JLabel(" ");
        final JTable table = new JTable();
        
        Thread  runThread = new Thread("<itemsets>") {
            public void run() {
                String query = ((DBLoader)(fr.getTR())).getSQLQuery();
                
                // Construction de restrictions pour la clause where de la requete SQL
                String where = "";
                for (int i=0; i<items.length; i++) {
                    String []tmp = DBLoader.getAttributeTable(items[i].getClasse());
                    where += "`"+tmp[1]+"`.`"+tmp[0]+"`='"+items[i].getValue()+"' and ";
                }
                
                // On ajoute les nouvelles restrictions
                StringBuffer buf = new StringBuffer(query);
                int i = query.lastIndexOf("where");
                buf.insert(i+6, where);
                query = buf.toString();
                
                // Excution de la requte et affichage du rsultat dans la table
                Database db = fr.getFileDBPanel().getDB();
                db.executeSQLQueryNoEx(query);
                ResultSetTableModel model = new CachingResultSetTableModel(db.getResultSet());
                table.setModel(model);
                lblStatus.setText(" " + table.getRowCount() + " enregistrements");
            }
        };
        runThread.setPriority(Thread.MIN_PRIORITY);
        runThread.start();
        
        contentPane.add(new JScrollPane(table), BorderLayout.CENTER);
        contentPane.add(lblStatus, BorderLayout.SOUTH);
    }
}
