/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 *  FileDBPanel.java
 *  2003-2004 Darnet Camille, Vouriot Thierry
 *  camsss@ifrance.com, yeri@fr.st
 *  http://www.yeri.fr.st
 *
 */
package merja.gui;

import java.io.*;
import java.util.*;
import java.awt.event.*;
import java.awt.*;
import javax.swing.*;
import javax.swing.table.*;
import merja.*;
import merja.images.Icons;
import merja.data.*;
import merja.associations.*;
import merja.db.*;

/** Panel permettant d'ouvrir un fichier de base de donnes et d'en afficher les
 * items ou les attributs
 */
public class FileDBPanel extends MerjaPanel {
    /** Type base de donnes */
    public final static int DB = 1;
    /** Type fichier */
    public final static int FILE = 2;
    private final JTable m_attributesTable = new JTable();
    private final JTextField m_fileText = new JTextField();
    private final JTextField m_dbText = new JTextField();
    private TransactionsReader m_transactionsReader = null;
    private final JLabel m_nbAttributsLbl = new JLabel("Nb attributs : -");
    private final JLabel m_nbTransLbl = new JLabel("Nb transactions : -");
    private final JLabel m_nameLbl = new JLabel("Nom : -");
    private final JLabel m_nbItemsLbl = new JLabel("Nb items : -");
    private final JLabel m_nbItemsDiffLbl = new JLabel("Nb items diff : -");
    private AlgoInterface m_algoInterface;
    private final JFileChooser m_fileChooser = new JFileChooser(Merja.getMerjaHome());
    private DBChooserDialog m_dbChooser;
    private int m_typeData = FILE;
    
    /** Constructeur */
    FileDBPanel() {
        this.setLayout(new BorderLayout());
        
        final JButton openFileBtn = new JButton("Ouvrir fichier");
        final JButton openBddBtn = new JButton("Ouvrir BDD");
        openBddBtn.setPreferredSize(new Dimension(100, 25));
        openFileBtn.setPreferredSize(new Dimension(100, 25));
        
        m_attributesTable.setColumnSelectionAllowed(false);
        //m_attributesTable.setDragEnabled(false);
        
        JPanel panelNorth = new JPanel();
        JPanel panelCenter = new JPanel();
        JPanel panelEast = new JPanel();
        
        panelNorth.setLayout(new BorderLayout(5,5));
        panelCenter.setLayout(new BorderLayout());
        panelEast.setLayout(new BoxLayout(panelEast,BoxLayout.Y_AXIS));
        
        JPanel panelBdd = new JPanel(new BorderLayout(5,5));
        JPanel panelFile = new JPanel(new BorderLayout(5,5));
        
        m_fileText.setEditable(false);
        m_fileText.setBackground(panelCenter.getBackground());
        m_fileText.setSize(20, 400);
        m_dbText.setEditable(false);
        m_dbText.setBackground(panelCenter.getBackground());
        m_dbText.setSize(20, 400);
        
        panelFile.add(openFileBtn,BorderLayout.WEST);
        panelFile.add(m_fileText,BorderLayout.CENTER);
        
        panelBdd.add(openBddBtn,BorderLayout.WEST);
        panelBdd.add(m_dbText,BorderLayout.CENTER);
        
        panelNorth.add(panelFile,BorderLayout.NORTH);
        panelNorth.add(panelBdd,BorderLayout.CENTER);
        
        panelCenter.add(new JScrollPane(m_attributesTable),BorderLayout.CENTER);
        
        panelCenter.setBorder(BorderFactory.createCompoundBorder(
        BorderFactory.createTitledBorder("Attributs et items"),
        BorderFactory.createEmptyBorder(0, 0, 0, 0)
        ));
        
        JPanel panelStats = new JPanel();
        panelStats.setPreferredSize(new Dimension(150, 30));
        panelStats.setLayout(new BoxLayout(panelStats,BoxLayout.Y_AXIS));
        
        panelStats.add(m_nameLbl);
        panelStats.add(m_nbTransLbl);
        panelStats.add(m_nbItemsLbl);
        panelStats.add(m_nbItemsDiffLbl);
        panelStats.add(m_nbAttributsLbl);
        
        panelEast.add(panelStats);
        panelEast.setBorder(BorderFactory.createCompoundBorder(
        BorderFactory.createTitledBorder("Statistiques"),
        BorderFactory.createEmptyBorder(0, 5, 5, 5)
        ));
        
        panelNorth.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 2));
        
        this.add(panelCenter,BorderLayout.CENTER);
        this.add(panelNorth,BorderLayout.NORTH);
        this.add(panelEast,BorderLayout.EAST);
        
        openFileBtn.addActionListener(Merja.getAction("open"));
        openBddBtn.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                System.out.println("Action: opendb");
                open(DB);
            }
        });
    }
    
    /** Change la fentre principale de l'application
     * @param frame fentre principale de l'application
     */
    public void setMerjaFrame(MerjaFrame frame) {
        m_merjaFrame = frame;
        m_dbChooser = new DBChooserDialog(m_merjaFrame);
    }
    
    /** Ouverture d'un fichier ou d'une base de donnes
     * @param type <CODE>DB</CODE> pour une base de donnes, <CODE>FILE</CODE> pour une fichier
     */
    public void open(final int type) {
        m_fileChooser.resetChoosableFileFilters();
        boolean ok = false;
        final ExampleFileFilter filter = new ExampleFileFilter();
        if (type == FILE) {
            filter.addExtension("arff");
            filter.addExtension("csv");
            filter.setDescription("WEKA & CVS fichiers");
            m_fileChooser.setFileFilter(filter);
            ok = (m_fileChooser.showOpenDialog(m_merjaFrame) == JFileChooser.APPROVE_OPTION);
        }
        else {
            ok = (m_dbChooser.showOpenDialog(m_merjaFrame) == JFileChooser.APPROVE_OPTION);
        }
        
        if (ok) {
            if (m_runThread == null) {
                m_runThread = new Thread("<chargement>") {
                    public void run() {
                        m_logPanel.taskStarted();
                        m_merjaFrame.getChrono().start();
                        boolean ok = false;
                        if (type == FILE)
                            ok = openFile(m_fileChooser.getSelectedFile().getPath());
                        else ok = openBdd(m_dbChooser.getDB(), m_dbChooser.getSqlQuery());
                        
                        if (ok) {
                            m_merjaFrame.setAlgoInterface(new AlgoInterface(m_merjaFrame.getTR(), m_logPanel));
                            m_merjaFrame.getChrono().stop();
                            m_logPanel.logMessage("Base de donnes charge : " + m_merjaFrame.getTR().getRelationName() + " (" + m_merjaFrame.getChrono().timeElapsed() + "ms)");
                            m_logPanel.statusMessage("OK");
                            m_merjaFrame.activePanelsAndButtons(true, true, true, false);
                            m_merjaFrame.getDataPanel().cleanAll();
                            m_merjaFrame.getAssociationRulesPanel().cleanAll();
                            m_merjaFrame.getItemsetsPanel().cleanAll();
                            m_merjaFrame.getItemsetsPanel().setSupportLabel();
                            m_merjaFrame.getDataPanel().setRowSlider(m_merjaFrame.getTR().getNbTransactions());
                        }
                        m_merjaFrame.getChrono().stop();
                        m_logPanel.taskFinished();
                        m_runThread = null;
                    }
                };
                m_runThread.setPriority(Thread.MIN_PRIORITY);
                m_runThread.start();
            }
        }
    }
    
    private boolean openFile(String file) {
        boolean attributs;
        m_logPanel.statusMessage("Chargement base de donnes ...");
        
        String extension = Utilities.getFileExtension(file);
        try {
            if (extension.equalsIgnoreCase(".arff")) {
                m_transactionsReader = new ArffLoader(file);
                attributs = true;
            }
            else if (extension.equalsIgnoreCase(".csv")) {
                m_transactionsReader = new CsvLoader(file);
                attributs = false;
            }
            else {
                m_logPanel.logMessage("Echec chargement <" + file + "> : extension invalide");
                m_logPanel.statusMessage("Erreur");
                return false;
            }
        }
        catch (Exception e) {
            System.out.println("FileDBPanel::openFile : "+e);
            m_logPanel.logMessage("Echec chargement <" + file + ">");
            m_logPanel.logMessage("Erreur : " +e.getMessage());
            m_logPanel.statusMessage("Erreur");
            return false;
        }
        m_dbText.setText("");
        m_fileText.setText(file);
        m_nameLbl.setText("Nom : " + m_transactionsReader.getRelationName());
        m_nbItemsDiffLbl.setText("Nb items diff : " + m_transactionsReader.getNbItemsDiff());
        m_nbTransLbl.setText("Nb transactions : " + m_transactionsReader.getNbTransactions());
        m_nbItemsLbl.setText("Nb items : " + m_transactionsReader.getNbItems());
        
        if (extension.equalsIgnoreCase(".arff"))
            initTableWithAttributs();
        else if (extension.equalsIgnoreCase(".csv"))
            initTableWithItems();
        
        m_typeData = FILE;
        return true;
    }
    
    private void initTableWithAttributs() {
        m_nbAttributsLbl.setText("Nb attributs : " + m_transactionsReader.getNbAttributes());
        String []col = {"classe", "valeurs"};
        Item[] items = m_transactionsReader.getAttributes();
        StringBuffer tab[][] = new StringBuffer[m_transactionsReader.getNbAttributes()][2];
        String classe = null;
        int width = 0;
        int j = -1;
        for (int i=0; i<items.length; i++) {
            if (classe == null || !classe.equals(items[i].getClasse())) {
                j++; if (j == tab.length) break;
                tab[j][0] = new StringBuffer(items[i].getClasse());
                tab[j][1] = new StringBuffer("");
                if (tab[j][0].length() > width) width = tab[j][0].length();
            }
            tab[j][1].append(items[i].getValue()+"  ");
            classe = items[i].getClasse();
        }
        m_attributesTable.setModel(new ItemsTableModel(tab,col));
        
        this.m_attributesTable.getColumn("classe").setPreferredWidth(width*7);
        this.m_attributesTable.getColumn("classe").setMaxWidth(width*7);
    }
    
    private void initTableWithItems() {
        m_nbAttributsLbl.setText("Nb attributs : -");
        String []col = {"items diffrents"};
        Item[] items = m_transactionsReader.getAttributes();
        StringBuffer tab[][] = new StringBuffer[items.length][1];
        
        for (int i=0; i<items.length; i++) {
            tab[i][0] = new StringBuffer(items[i].getValue());
        }
        
        TableModel tm = new ItemsTableModel(tab,col);
        m_attributesTable.setModel(tm);
    }
    
    private boolean openBdd(Database db, String sqlQuery) {
        try {
            m_transactionsReader = new DBLoader(db, sqlQuery, this.m_dbChooser.getName());
        }
        catch (Exception e) {
            
            // Efface tout dans le cas d'une erreur
            m_merjaFrame.getDataPanel().cleanAll();
            m_merjaFrame.getAssociationRulesPanel().cleanAll();
            m_merjaFrame.getFileDBPanel().cleanAll();
            m_merjaFrame.getItemsetsPanel().cleanAll();
            
            m_merjaFrame.setAlgoInterface(null);
            
            m_merjaFrame.activePanelsAndButtons(true, false, false, false);
            m_merjaFrame.setTabPanel(m_merjaFrame.getFileDBPanel());
            
            System.out.println("FileDBPanel::openBdd : "+e);
            m_logPanel.logMessage("Echec chargement <" + db.getURL() + ">");
            m_logPanel.logMessage("Erreur : " +e.getMessage());
            m_logPanel.statusMessage("Erreur");
            
            return false;
        }
        m_fileText.setText("");
        m_dbText.setText(db.getURL() + " : " + sqlQuery);
        m_nameLbl.setText("Nom : " + m_transactionsReader.getRelationName());
        m_nbItemsDiffLbl.setText("Nb items diff : " + m_transactionsReader.getNbItemsDiff());
        m_nbTransLbl.setText("Nb transactions : " + m_transactionsReader.getNbTransactions());
        m_nbItemsLbl.setText("Nb items : " + m_transactionsReader.getNbItems());
        
        initTableWithAttributs();
        
        m_typeData = DB;
        return true;
    }
    
    /** Retourne le <CODE>TransactionsReader</CODE> permettant la lecture de la base de
     * donnes
     * @return interface permettant la lecture de la base de donnes
     */
    public TransactionsReader getTR() {
        return this.m_transactionsReader;
    }
    
    /** Retourne une classe permettant d'effectuer les traitements de recherche d'items
     * et de rgles d'association
     * @return classe permettant d'effectuer les traitements
     */
    public AlgoInterface getAlgoInterface() {
        return this.m_algoInterface;
    }
    
    /** Change la classe permettant d'effectuer les traitements de recherche d'items
     * et de rgles d'association
     * @param af nouvelle classe permettant d'effectuer les traitements
     */
    public void setAlgoInterface(AlgoInterface af) {
        this.m_algoInterface = af;
    }
    
    /** Retourne l'objet <CODE>Database</CODE> si les donnes proviennent d'une base de donnes
     * @return objet <CODE>Database</CODE> ou NULL si les donnes proviennent d'un fichier
     */
    public Database getDB() {
        if (m_typeData == DB)
            return this.m_dbChooser.getDB();
        else return null;
    }
    
    /** Efface et rinitialise tous les composants du panel */
    public void cleanAll() {
        m_attributesTable.setModel(new DefaultTableModel());
        m_fileText.setText("");
        m_dbText.setText("");
        m_transactionsReader = null;
        m_nbAttributsLbl.setText("Nb attributs : -");
        m_nbTransLbl.setText("Nb transactions : -");
        m_nameLbl.setText("Nom : -");
        m_nbItemsLbl.setText("Nb items : -");
        m_nbItemsDiffLbl.setText("Nb items diff : -");
        m_algoInterface = null;
        if (m_dbChooser.getDB() != null) m_dbChooser.getDB().closeRs();
    }
    
    /** Retourne le type de donnes soit DB pour des donnes en provenance d'une base de donnes
     *  soit FILE pour des donnes en provenance d'un fichier
     * @return type de donnes (DB ou FILE)
     */
    public int getDataType() {
        return this.m_typeData;
    }
}
