/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 *  FileDBPanel.java
 *  2003-2004 Darnet Camille, Vouriot Thierry
 *  camsss@ifrance.com, yeri@fr.st
 *  http://www.yeri.fr.st
 *
 */
package merja.gui;

import javax.swing.*;
import javax.swing.table.*;

/** <CODE>TableModel</CODE> permettant d'afficher les attributs et les items dans
 * une <CODE>JTable</CODE>
 */
public class ItemsTableModel extends AbstractTableModel {
    
    private StringBuffer [][] m_items;
    private String [] m_columnName;
    
    /** Constructeur
     * @param items tableau d'items
     * @param columnName tableau avec le nom des colonnes
     */    
    public ItemsTableModel(StringBuffer items[][], String columnName[]) {
        m_items = items;
        m_columnName = columnName;
    }
    
    /** Retourne le nombre de lignes
     * @return nombre de lignes
     */    
    public int getRowCount() {
        return m_items.length;
    }
    
    /** Retourne le nombre de colonnes
     * @return nombre de colonnes
     */    
    public int getColumnCount() {
        return m_columnName.length;
    }
    
    /** Retourne l'item  une certaine ligne et une certaine colonne
     * @param row numro de la lignes
     * @param column numro de la colonne
     * @return items  la colonne <I>column</I> et la ligne <I>row</I>
     */    
    public Object getValueAt(int row, int column) {
        // genere une exception de temps en temps je sais pas pourkoi
        // il faut donc rajouter cette ligne
        if (column == getColumnCount()) column--;
        return m_items[row][column].toString();
    }
    
    /** Retourne le nom d'une colonne
     * @param columnIndex index de la colonne
     * @return nom de la colonne
     */    
    public String getColumnName(int columnIndex) {
        return m_columnName[columnIndex];
    }

}
