/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 *  DataPanel.java
 *  2003-2004 Darnet Camille, Vouriot Thierry
 *  camsss@ifrance.com, yeri@fr.st
 *  http://www.yeri.fr.st
 *
 */
package merja.gui;

import java.awt.event.*;
import java.awt.*;
import javax.swing.*;
import merja.*;
import merja.images.Icons;
import merja.associations.*;

/** Panels utiliss dans le <CODE>MerjaTabbedPane</CODE> */
public class MerjaPanel extends JPanel {
    
    /** Rfrence vers le panel de log */    
    protected LogPanel m_logPanel = null;
    
    /** Fentre principale de Merja */    
    protected MerjaFrame m_merjaFrame = null;
    
    /** Thread pour effectuer des traitements */    
    protected Thread m_runThread = null;
    
    /** Constructeur */    
    protected MerjaPanel() { }
    
    /** Constructeur 
     * @param logPanel nouveau panel de log
     * @param frame fentre principale de l'application
     */    
    protected MerjaPanel(MerjaFrame frame, LogPanel logPanel) { 
        m_logPanel = logPanel;
        m_merjaFrame = frame;
    }
    
    /** Change le panel de log
     * @param logPanel nouveau panel de log
     */    
    public void setLogPanel(LogPanel logPanel) {
        m_logPanel = logPanel;
    }
    
    /** Change la fentre principale de l'application
     * @param frame fentre principale de l'application
     */    
    public void setMerjaFrame(MerjaFrame frame) {
        m_merjaFrame = frame;
    }
    
    /** Test si le panel est en train d'effectuer un traitement
     * @return vrai si le panel effectue un traitement
     */    
    public boolean threadIsRunning() {
        if (m_runThread != null && m_runThread.isAlive())
            return true;
        return false;
        
    }
    
    /** Arrte le traitement en cours du panel */    
    public void stopThread() {
        if (m_runThread != null) {
            m_logPanel.logMessage("Arrt du traitement : thread " + m_runThread.getName());
            m_runThread.interrupt();
            m_runThread.stop();
            m_runThread = null;
            //m_merjaFrame.setAlgoInterface(new AlgoInterface(m_merjaFrame.getTR()));
        }
        // si il n'a plus aucun thread qui tourne on stop le taskmonitor
        if (!m_merjaFrame.getFileDBPanel().threadIsRunning() &&
        !m_merjaFrame.getDataPanel().threadIsRunning() &&
        !m_merjaFrame.getItemsetsPanel().threadIsRunning() &&
        !m_merjaFrame.getAssociationRulesPanel().threadIsRunning())
            m_logPanel.taskFinished();
    }
}