/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 *  ResultSetTableModel.java
 *  2003-2004 Darnet Camille, Vouriot Thierry
 *  camsss@fr.st, yeri@fr.st
 *  http://www.yeri.fr.st
 *
 */
package merja.gui;

import java.awt.*;
import java.awt.event.*;
import java.sql.*;
import java.util.*;
import javax.swing.*;
import javax.swing.table.*;

/* this class is the base class for the scrolling and the
   caching result set table model. It stores the result set
   and its metadata.
 */
public abstract class ResultSetTableModel extends AbstractTableModel
{  public ResultSetTableModel(ResultSet aResultSet)
   {  rs = aResultSet;
      try
      {  rsmd = rs.getMetaData();
      }
      catch(SQLException e)
      {  System.out.println("Error " + e);
      }
   }
   
   public String getColumnName(int c)
   {  try
      {  return rsmd.getColumnName(c + 1);
      }
      catch(SQLException e)
      {  System.out.println("Error " + e);
         return "";
      }
   }
   
   public int getColumnCount()
   {  try
      {  return rsmd.getColumnCount();
      }
      catch(SQLException e)
      {  System.out.println("Error " + e);
         return 0;
      }
   }
   
   protected ResultSet getResultSet()
   {  return rs;
   }
   
   private ResultSet rs;
   private ResultSetMetaData rsmd;
}

/* this class uses a scrolling cursor, a JDBC 2 feature
 */
class ScrollingResultSetTableModel extends ResultSetTableModel
{  public ScrollingResultSetTableModel(ResultSet aResultSet)
   {  super(aResultSet);
   }
   
   public Object getValueAt(int r, int c)
   {  try
      {  ResultSet rs = getResultSet();
         rs.absolute(r + 1);
         return rs.getObject(c + 1);
      }
      catch(SQLException e)
      {  System.out.println("Error " + e);
         return null;
      }
   }
   
   public int getRowCount()
   {  try
      {  ResultSet rs = getResultSet();
         rs.last();
         return rs.getRow();
      }
      catch(SQLException e)
      {  System.out.println("Error " + e);
         return 0;
      }
   }
}

/* this class caches the result set data; it can be used
   if scrolling cursors are not supported
 */
class CachingResultSetTableModel extends ResultSetTableModel
{  public CachingResultSetTableModel(ResultSet aResultSet)
   {  super(aResultSet);
      try
      {  cache = new ArrayList();
         int cols = getColumnCount();
         ResultSet rs = getResultSet();
         
         /* place all data in an array list of Object[] arrays
            We don't use an Object[][] because we don't know
            how many rows are in the result set
          */
         
         while (rs.next())
         {  Object[] row = new Object[cols];
            for (int j = 0; j < row.length; j++)
                row[j] = rs.getObject(j + 1);
            cache.add(row);
         }
      }
      catch(SQLException e)
      {  System.out.println("Error " + e);
      }
   }
   
   public Object getValueAt(int r, int c)
   {  if (r < cache.size())
          return ((Object[])cache.get(r))[c];
      else
          return null;
   }
   
   public int getRowCount()
   {  return cache.size();
   }
   
   private ArrayList cache;
}